@@ -89,6 +89,9 @@
  *	directly -- and assumed always to succeed.
  */
 
+/* $Id: zend_strtod.c,v 1.17.2.2.2.12 2007/07/23 16:17:10 jani Exp $ */
+
+#include <zend_operators.h>
 #include <zend_strtod.h>
 
 #ifdef ZTS
@@ -271,9 +274,9 @@ BEGIN_EXTERN_C()
 #endif
 
 /* The following definition of Storeinc is appropriate for MIPS processors.
- *  * An alternative that might be better on some machines is
- *   * #define Storeinc(a,b,c) (*a++ = b << 16 | c & 0xffff)
- *    */
+ * An alternative that might be better on some machines is
+ * #define Storeinc(a,b,c) (*a++ = b << 16 | c & 0xffff)
+ */
 #if defined(IEEE_LITTLE_ENDIAN) + defined(VAX) + defined(__arm__)
 #define Storeinc(a,b,c) (((unsigned short *)a)[1] = (unsigned short)b, \
 		((unsigned short *)a)[0] = (unsigned short)c, a++)
@@ -410,7 +413,10 @@ struct Bigint {
 
 typedef struct Bigint Bigint;
 
+/* static variables, multithreading fun! */
 static Bigint *freelist[Kmax+1];
+static Bigint *p5s;
+
 static void destroy_freelist(void);
 
 #ifdef ZTS
@@ -456,12 +462,20 @@ static Bigint * Balloc(int k)
 	int x;
 	Bigint *rv;
 
+	if (k > Kmax) {
+		zend_error(E_ERROR, "Balloc() allocation exceeds list boundary");
+	}
+
 	_THREAD_PRIVATE_MUTEX_LOCK(dtoa_mutex);
 	if ((rv = freelist[k])) {
 		freelist[k] = rv->next;
 	} else {
 		x = 1 << k;
 		rv = (Bigint *)MALLOC(sizeof(Bigint) + (x-1)*sizeof(Long));
+		if (!rv) {
+			_THREAD_PRIVATE_MUTEX_UNLOCK(dtoa_mutex);
+			zend_error(E_ERROR, "Balloc() failed to allocate memory");
+		}
 		rv->k = k;
 		rv->maxwds = x;
 	}
@@ -754,28 +768,25 @@ static Bigint * s2b (CONST char *s, int nd0, int nd, ULong y9)
 	return b;
 }
 
-
-static Bigint *p5s;
-
 static Bigint * pow5mult(Bigint *b, int k)
 {
 	Bigint *b1, *p5, *p51;
 	int i;
 	static int p05[3] = { 5, 25, 125 };
 
+	_THREAD_PRIVATE_MUTEX_LOCK(pow5mult_mutex);
 	if ((i = k & 3)) {
 		b = multadd(b, p05[i-1], 0);
 	}
 
 	if (!(k >>= 2)) {
+		_THREAD_PRIVATE_MUTEX_UNLOCK(pow5mult_mutex);
 		return b;
 	}
 	if (!(p5 = p5s)) {
 		/* first time */
-		_THREAD_PRIVATE_MUTEX_LOCK(pow5mult_mutex);
 		p5 = p5s = i2b(625);
 		p5->next = 0;
-		_THREAD_PRIVATE_MUTEX_UNLOCK(pow5mult_mutex);
 	}
 	for(;;) {
 		if (k & 1) {
@@ -787,15 +798,14 @@ static Bigint * pow5mult(Bigint *b, int k)
 			break;
 		}
 		if (!(p51 = p5->next)) {
-			_THREAD_PRIVATE_MUTEX_LOCK(pow5mult_mutex);
 			if (!(p51 = p5->next)) {
 				p51 = p5->next = mult(p5,p5);
 				p51->next = 0;
 			}
-			_THREAD_PRIVATE_MUTEX_UNLOCK(pow5mult_mutex);
 		}
 		p5 = p51;
 	}
+	_THREAD_PRIVATE_MUTEX_UNLOCK(pow5mult_mutex);
 	return b;
 }
 
@@ -1700,7 +1710,14 @@ ZEND_API char * zend_dtoa(double _d, int mode, int ndigits, int *decpt, int *sig
 					if (value(d) > 0.5 + value(eps))
 						goto bump_up;
 					else if (value(d) < 0.5 - value(eps)) {
-						while(*--s == '0');
+						/* cut ALL traling zeros only if the number of chars is greater than precision 
+						 * otherwise cut only extra zeros
+						 */
+						if (k < ndigits) {
+							while(*--s == '0' && (s - s0) > k);
+						} else {
+							while(*--s == '0');
+						}
 						s++;
 						goto ret1;
 					}
@@ -2590,6 +2607,34 @@ ZEND_API double zend_hex_strtod(const char *str, char **endptr)
 	return value;
 }
 
+ZEND_API double zend_oct_strtod(const char *str, char **endptr)
+{
+	const char *s = str;
+	char c;
+	double value = 0;
+	int any = 0;
+
+	/* skip leading zero */
+	s++;
+
+	while ((c = *s++)) {
+		if (c > '7') {
+			/* break and return the current value if the number is not well-formed
+			 * that's what Linux strtol() does 
+			 */
+			break;
+		}
+		value = value * 8 + c - '0';
+		any = 1;
+	}
+
+	if (endptr != NULL) {
+		*endptr = (char *)(any ? s - 1 : str);
+	}
+
+	return value;
+}
+
 /*
  * Local variables:
  * tab-width: 4
