@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: interface.c,v 1.62.2.14.2.22 2007/01/19 18:03:33 tony2001 Exp $ */
+/* $Id: interface.c,v 1.62.2.14.2.27 2007/07/04 13:34:23 tony2001 Exp $ */
 
 #define ZEND_INCLUDE_FULL_WINDOWS_HEADERS
 
@@ -173,7 +173,7 @@ static void _php_curl_close(zend_rsrc_list_entry *rsrc TSRMLS_DC);
 			php_curl_ret(__ret);											\
 		} 													\
 															\
-		if (php_memnstr(str, tmp_url->path, strlen(tmp_url->path), str + len)) {				\
+		if (!php_memnstr(str, tmp_url->path, strlen(tmp_url->path), str + len)) {				\
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "URL '%s' contains unencoded control characters.", str);	\
 			php_url_free(tmp_url); 																\
 			php_curl_ret(__ret);											\
@@ -387,6 +387,9 @@ PHP_MINIT_FUNCTION(curl)
 	REGISTER_CURL_CONSTANT(CURLOPT_PROXYUSERPWD);
 	REGISTER_CURL_CONSTANT(CURLOPT_RANGE);
 	REGISTER_CURL_CONSTANT(CURLOPT_TIMEOUT);
+#ifdef CURLOPT_TIMEOUT_MS
+	REGISTER_CURL_CONSTANT(CURLOPT_TIMEOUT_MS);
+#endif
 	REGISTER_CURL_CONSTANT(CURLOPT_POSTFIELDS);
 	REGISTER_CURL_CONSTANT(CURLOPT_REFERER);
 	REGISTER_CURL_CONSTANT(CURLOPT_USERAGENT);
@@ -430,6 +433,9 @@ PHP_MINIT_FUNCTION(curl)
 	REGISTER_CURL_CONSTANT(CURLOPT_RANDOM_FILE);
 	REGISTER_CURL_CONSTANT(CURLOPT_EGDSOCKET);
 	REGISTER_CURL_CONSTANT(CURLOPT_CONNECTTIMEOUT);
+#ifdef CURLOPT_CONNECTTIMEOUT_MS
+	REGISTER_CURL_CONSTANT(CURLOPT_CONNECTTIMEOUT_MS);
+#endif
 	REGISTER_CURL_CONSTANT(CURLOPT_SSL_VERIFYPEER);
 	REGISTER_CURL_CONSTANT(CURLOPT_CAINFO);
 	REGISTER_CURL_CONSTANT(CURLOPT_CAPATH);
@@ -471,10 +477,13 @@ PHP_MINIT_FUNCTION(curl)
  	REGISTER_CURL_CONSTANT(CURLAUTH_ANYSAFE);
 #endif
 
-#if LIBCURL_VERSION_NUM > 0x070a06 /* CURLOPT_PROXYAUTH is available since curl 7.10.7 */
+#if LIBCURL_VERSION_NUM > 0x070a06 /* CURLOPT_PROXYAUTH & CURLOPT_FTP_CREATE_MISSING_DIRS are available since curl 7.10.7 */
 	REGISTER_CURL_CONSTANT(CURLOPT_PROXYAUTH);
+	REGISTER_CURL_CONSTANT(CURLOPT_FTP_CREATE_MISSING_DIRS);
 #endif
 
+	REGISTER_CURL_CONSTANT(CURLOPT_PRIVATE);
+
 	/* Constants effecting the way CURLOPT_CLOSEPOLICY works */
 	REGISTER_CURL_CONSTANT(CURLCLOSEPOLICY_LEAST_RECENTLY_USED);
 	REGISTER_CURL_CONSTANT(CURLCLOSEPOLICY_LEAST_TRAFFIC);
@@ -504,6 +513,7 @@ PHP_MINIT_FUNCTION(curl)
 	REGISTER_CURL_CONSTANT(CURLINFO_REDIRECT_TIME);
 	REGISTER_CURL_CONSTANT(CURLINFO_REDIRECT_COUNT);
 	REGISTER_CURL_CONSTANT(CURLINFO_HEADER_OUT);
+	REGISTER_CURL_CONSTANT(CURLINFO_PRIVATE);
 
 	/* cURL protocol constants (curl_version) */
 	REGISTER_CURL_CONSTANT(CURL_VERSION_IPV6);
@@ -715,7 +725,9 @@ static size_t curl_write(char *data, size_t size, size_t nmemb, void *ctx)
 		case PHP_CURL_FILE:
 			return fwrite(data, size, nmemb, t->fp);
 		case PHP_CURL_RETURN:
-			smart_str_appendl(&t->buf, data, (int) length);
+			if (length > 0) {
+				smart_str_appendl(&t->buf, data, (int) length);
+			}
 			break;
 		case PHP_CURL_USER: {
 			zval **argv[2];
@@ -854,7 +866,7 @@ static size_t curl_write_header(char *data, size_t size, size_t nmemb, void *ctx
 		case PHP_CURL_STDOUT:
 			/* Handle special case write when we're returning the entire transfer
 			 */
-			if (ch->handlers->write->method == PHP_CURL_RETURN) {
+			if (ch->handlers->write->method == PHP_CURL_RETURN && length > 0) {
 				smart_str_appendl(&ch->handlers->write->buf, data, (int) length);
 			} else {
 				PHPWRITE(data, length);
@@ -1221,6 +1233,9 @@ static int _php_curl_setopt(php_curl *ch, long option, zval **zvalue, zval *retu
 		 case CURLOPT_MUTE:
 #endif
 		case CURLOPT_TIMEOUT:
+#ifdef CURLOPT_TIMEOUT_MS
+		case CURLOPT_TIMEOUT_MS:
+#endif
 		case CURLOPT_FTP_USE_EPSV:
 		case CURLOPT_LOW_SPEED_LIMIT:
 		case CURLOPT_SSLVERSION:
@@ -1237,6 +1252,9 @@ static int _php_curl_setopt(php_curl *ch, long option, zval **zvalue, zval *retu
 		case CURLOPT_FRESH_CONNECT:
 		case CURLOPT_FORBID_REUSE:
 		case CURLOPT_CONNECTTIMEOUT:
+#ifdef CURLOPT_CONNECTTIMEOUT_MS
+		case CURLOPT_CONNECTTIMEOUT_MS:
+#endif
 		case CURLOPT_SSL_VERIFYHOST:
 		case CURLOPT_SSL_VERIFYPEER:
 		case CURLOPT_DNS_USE_GLOBAL_CACHE:
@@ -1252,8 +1270,9 @@ static int _php_curl_setopt(php_curl *ch, long option, zval **zvalue, zval *retu
 #if LIBCURL_VERSION_NUM > 0x070a05 /* CURLOPT_HTTPAUTH is available since curl 7.10.6 */
 		case CURLOPT_HTTPAUTH:
 #endif
-#if LIBCURL_VERSION_NUM > 0x070a06 /* CURLOPT_PROXYAUTH is available since curl 7.10.7 */
+#if LIBCURL_VERSION_NUM > 0x070a06 /* CURLOPT_PROXYAUTH & CURLOPT_FTP_CREATE_MISSING_DIRS are available since curl 7.10.7 */
 		case CURLOPT_PROXYAUTH:
+		case CURLOPT_FTP_CREATE_MISSING_DIRS:
 #endif
 
 #if LIBCURL_VERSION_NUM >= 0x070c02
@@ -1283,6 +1302,7 @@ static int _php_curl_setopt(php_curl *ch, long option, zval **zvalue, zval *retu
 			}
 			error = curl_easy_setopt(ch->cp, option, Z_LVAL_PP(zvalue));
 			break;
+		case CURLOPT_PRIVATE:
 		case CURLOPT_URL:
 		case CURLOPT_PROXY:
 		case CURLOPT_USERPWD:
@@ -1786,6 +1806,7 @@ PHP_FUNCTION(curl_getinfo)
 	} else {
 		option = Z_LVAL_PP(zoption);
 		switch (option) {
+			case CURLINFO_PRIVATE:
 			case CURLINFO_EFFECTIVE_URL: 
 			case CURLINFO_CONTENT_TYPE: {
  				char *s_code = NULL;
