@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: ftp.c,v 1.112.2.4.2.3 2007/01/01 09:36:01 sebastian Exp $ */
+/* $Id: ftp.c,v 1.112.2.4.2.9 2007/06/07 08:59:00 tony2001 Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -266,60 +266,57 @@ ftp_login(ftpbuf_t *ftp, const char *user, const char *pass TSRMLS_DC)
 			}
 				
 			if (ftp->resp != 334) {
-				ftp->use_ssl = 0;
+				return 0;
 			} else {
 				ftp->old_ssl = 1;
 				ftp->use_ssl_for_data = 1;
 			}
 		}
 		
-		/* now enable ssl if we still need to */
-		if (ftp->use_ssl) {
-			ctx = SSL_CTX_new(SSLv23_client_method());
-			if (ctx == NULL) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL context");
+		ctx = SSL_CTX_new(SSLv23_client_method());
+		if (ctx == NULL) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL context");
+			return 0;
+		}
+
+		SSL_CTX_set_options(ctx, SSL_OP_ALL);
+
+		ftp->ssl_handle = SSL_new(ctx);
+		if (ftp->ssl_handle == NULL) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL handle");
+			SSL_CTX_free(ctx);
+			return 0;
+		}
+
+		SSL_set_fd(ftp->ssl_handle, ftp->fd);
+
+		if (SSL_connect(ftp->ssl_handle) <= 0) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL/TLS handshake failed");
+			SSL_shutdown(ftp->ssl_handle);
+			return 0;
+		}
+
+		ftp->ssl_active = 1;
+
+		if (!ftp->old_ssl) {
+
+			/* set protection buffersize to zero */
+			if (!ftp_putcmd(ftp, "PBSZ", "0")) {
+				return 0;
+			}
+			if (!ftp_getresp(ftp)) {
 				return 0;
 			}
 
-			SSL_CTX_set_options(ctx, SSL_OP_ALL);
-
-			ftp->ssl_handle = SSL_new(ctx);
-			if (ftp->ssl_handle == NULL) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to create the SSL handle");
-				SSL_CTX_free(ctx);
+			/* enable data conn encryption */
+			if (!ftp_putcmd(ftp, "PROT", "P")) {
+				return 0;
+			}
+			if (!ftp_getresp(ftp)) {
 				return 0;
 			}
 			
-			SSL_set_fd(ftp->ssl_handle, ftp->fd);
-			
-			if (SSL_connect(ftp->ssl_handle) <= 0) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "SSL/TLS handshake failed");
-				SSL_shutdown(ftp->ssl_handle);
-				return 0;
-			}
-			
-			ftp->ssl_active = 1;
-			
-			if (!ftp->old_ssl) {
-				
-				/* set protection buffersize to zero */
-				if (!ftp_putcmd(ftp, "PBSZ", "0")) {
-					return 0;
-				}
-				if (!ftp_getresp(ftp)) {
-					return 0;
-				}
-					
-				/* enable data conn encryption */
-				if (!ftp_putcmd(ftp, "PROT", "P")) {
-					return 0;
-				}
-				if (!ftp_getresp(ftp)) {
-					return 0;
-				}
-				
-				ftp->use_ssl_for_data = (ftp->resp >= 200 && ftp->resp <=299);		
-			}
+			ftp->use_ssl_for_data = (ftp->resp >= 200 && ftp->resp <=299);		
 		}
 	}
 #endif
@@ -589,11 +586,7 @@ ftp_chmod(ftpbuf_t *ftp, const int mode, const char *filename, const int filenam
 		return 0;
 	}
 
-	if (!(buffer = emalloc(32 + filename_len + 1))) {
-		return 0;
-	}
-
-	sprintf(buffer, "CHMOD %o %s", mode, filename);
+	spprintf(&buffer, 0, "CHMOD %o %s", mode, filename);
 
 	if (!ftp_putcmd(ftp, "SITE", buffer)) {
 		efree(buffer);
@@ -813,7 +806,7 @@ ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t type,
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "PHP cannot handle files greater than 2147483647 bytes.");
 			goto bail;
 		}
-		sprintf(arg, "%u", resumepos);
+		snprintf(arg, sizeof(arg), "%u", resumepos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -910,7 +903,7 @@ ftp_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type, i
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "PHP cannot handle files with a size greater than 2147483647 bytes.");
 			goto bail;
 		}
-		sprintf(arg, "%u", startpos);
+		snprintf(arg, sizeof(arg), "%u", startpos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -1019,6 +1012,9 @@ ftp_mdtm(ftpbuf_t *ftp, const char *path)
 	/* figure out the GMT offset */
 	stamp = time(NULL);
 	gmt = php_gmtime_r(&stamp, &tmbuf);
+	if (!gmt) {
+		return -1;
+	}
 	gmt->tm_isdst = -1;
 
 	/* apply the GMT offset */
@@ -1103,19 +1099,25 @@ ftp_putcmd(ftpbuf_t *ftp, const char *cmd, const char *args)
 	int		size;
 	char		*data;
 
+	if (strpbrk(cmd, "\r\n")) {
+		return 0;
+	} 
 	/* build the output buffer */
 	if (args && args[0]) {
 		/* "cmd args\r\n\0" */
 		if (strlen(cmd) + strlen(args) + 4 > FTP_BUFSIZE) {
 			return 0;
 		}
-		size = sprintf(ftp->outbuf, "%s %s\r\n", cmd, args);
+		if (strpbrk(args, "\r\n")) {
+			return 0;
+		}
+		size = slprintf(ftp->outbuf, sizeof(ftp->outbuf), "%s %s\r\n", cmd, args);
 	} else {
 		/* "cmd\r\n\0" */
 		if (strlen(cmd) + 3 > FTP_BUFSIZE) {
 			return 0;
 		}
-		size = sprintf(ftp->outbuf, "%s\r\n", cmd);
+		size = slprintf(ftp->outbuf, sizeof(ftp->outbuf), "%s\r\n", cmd);
 	}
 
 	data = ftp->outbuf;
@@ -1441,7 +1443,7 @@ ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
 		char eprtarg[INET6_ADDRSTRLEN + sizeof("|x||xxxxx|")];
 		char out[INET6_ADDRSTRLEN];
 		inet_ntop(AF_INET6, &((struct sockaddr_in6*) sa)->sin6_addr, out, sizeof(out));
-		sprintf(eprtarg, "|2|%s|%hu|", out, ntohs(((struct sockaddr_in6 *) &addr)->sin6_port));
+		snprintf(eprtarg, sizeof(eprtarg), "|2|%s|%hu|", out, ntohs(((struct sockaddr_in6 *) &addr)->sin6_port));
 
 		if (!ftp_putcmd(ftp, "EPRT", eprtarg)) {
 			goto bail;
@@ -1459,7 +1461,7 @@ ftp_getdata(ftpbuf_t *ftp TSRMLS_DC)
 	/* send the PORT */
 	ipbox.ia[0] = ((struct sockaddr_in*) sa)->sin_addr;
 	ipbox.s[2] = ((struct sockaddr_in*) &addr)->sin_port;
-	sprintf(arg, "%u,%u,%u,%u,%u,%u", ipbox.c[0], ipbox.c[1], ipbox.c[2], ipbox.c[3], ipbox.c[4], ipbox.c[5]);
+	snprintf(arg, sizeof(arg), "%u,%u,%u,%u,%u,%u", ipbox.c[0], ipbox.c[1], ipbox.c[2], ipbox.c[3], ipbox.c[4], ipbox.c[5]);
 
 	if (!ftp_putcmd(ftp, "PORT", arg)) {
 		goto bail;
@@ -1716,7 +1718,7 @@ ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, ftptype_t typ
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "PHP cannot handle files greater than 2147483648 bytes.");
 			goto bail;
 		}
-		sprintf(arg, "%u", resumepos);
+		snprintf(arg, sizeof(arg), "%u", resumepos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
@@ -1834,7 +1836,7 @@ ftp_nb_put(ftpbuf_t *ftp, const char *path, php_stream *instream, ftptype_t type
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "PHP cannot handle files with a size greater than 2147483647 bytes.");
 			goto bail;
 		}
-		sprintf(arg, "%u", startpos);
+		snprintf(arg, sizeof(arg), "%u", startpos);
 		if (!ftp_putcmd(ftp, "REST", arg)) {
 			goto bail;
 		}
