@@ -120,8 +120,17 @@ gdImagePtr gdImageCreate (int sx, int sy)
 {
 	int i;
 	gdImagePtr im;
-	im = (gdImage *) gdMalloc(sizeof(gdImage));
-	memset(im, 0, sizeof(gdImage));
+
+	if (overflow2(sx, sy)) {
+		return NULL;
+	}
+
+	if (overflow2(sizeof(unsigned char *), sy)) {
+		return NULL;
+	}
+
+	im = (gdImage *) gdCalloc(1, sizeof(gdImage));
+
 	/* Row-major ever since gd 1.3 */
 	im->pixels = (unsigned char **) gdMalloc(sizeof(unsigned char *) * sy);
 	im->AA_opacity = (unsigned char **) gdMalloc(sizeof(unsigned char *) * sy);
@@ -162,6 +171,19 @@ gdImagePtr gdImageCreateTrueColor (int sx, int sy)
 {
 	int i;
 	gdImagePtr im;
+
+	if (overflow2(sx, sy)) {
+		return NULL;
+	}
+
+	if (overflow2(sizeof(unsigned char *), sy)) {
+		return NULL;
+	}
+	
+	if (overflow2(sizeof(int), sx)) {
+		return NULL;
+	}
+
 	im = (gdImage *) gdMalloc(sizeof(gdImage));
 	memset(im, 0, sizeof(gdImage));
 	im->tpixels = (int **) gdMalloc(sizeof(int *) * sy);
@@ -847,7 +869,9 @@ static void gdImageTileApply (gdImagePtr im, int x, int y)
 	srcy = y % gdImageSY(im->tile);
 	if (im->trueColor) {
 		p = gdImageGetTrueColorPixel(im->tile, srcx, srcy);
-		gdImageSetPixel(im, x, y, p);
+		if (p != gdImageGetTransparent (im->tile)) {
+			gdImageSetPixel(im, x, y, p);
+		}
 	} else {
 		p = gdImageGetPixel(im->tile, srcx, srcy);
 		/* Allow for transparency */
@@ -1015,74 +1039,93 @@ void gdImageAABlend (gdImagePtr im)
 	}
 }
 
+static void gdImageHLine(gdImagePtr im, int y, int x1, int x2, int col)
+{
+	if (im->thick > 1) {
+		int thickhalf = im->thick >> 1;
+		gdImageFilledRectangle(im, x1, y - thickhalf, x2, y + im->thick - thickhalf - 1, col);
+	} else {
+		if (x2 < x1) {
+			int t = x2;
+			x2 = x1;
+			x1 = t;
+		}
+
+		for (;x1 <= x2; x1++) {
+			gdImageSetPixel(im, x1, y, col);
+		}
+	}
+	return;
+}
+
+static void gdImageVLine(gdImagePtr im, int x, int y1, int y2, int col)
+{
+	if (im->thick > 1) {
+		int thickhalf = im->thick >> 1;
+		gdImageFilledRectangle(im, x - thickhalf, y1, x + im->thick - thickhalf - 1, y2, col);
+	} else {
+		if (y2 < y1) {
+			int t = y1;
+			y1 = y2;
+			y2 = t;
+		}
+
+		for (;y1 <= y2; y1++) {
+			gdImageSetPixel(im, x, y1, col);
+		}
+	}
+	return;
+}
 
 /* Bresenham as presented in Foley & Van Dam */
 void gdImageLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
 {
-	int t;
 	int dx, dy, incr1, incr2, d, x, y, xend, yend, xdirflag, ydirflag;
 	int wid;
 	int w, wstart;
 	int thick = im->thick;
 
+	if (color == gdAntiAliased)
+	{
+		/* 
+		   gdAntiAliased passed as color: use the much faster, much cheaper
+		   and equally attractive gdImageAALine implementation. That
+		   clips too, so don't clip twice.
+		   */
+		gdImageAALine(im, x1, y1, x2, y2, im->AA_color); 
+		return;
+	}
+
 	/* 2.0.10: Nick Atty: clip to edges of drawing rectangle, return if no points need to be drawn */
 	if (!clip_1d(&x1,&y1,&x2,&y2,gdImageSX(im)) || !clip_1d(&y1,&x1,&y2,&x2,gdImageSY(im))) {
 		return;
 	}
 
-	/* Vertical */
-	if (x1==x2) {
-		if (y2 < y1) {
-			t = y2;
-			y2 = y1;
-			y1 = t;
-		}
+	dx = abs (x2 - x1);
+	dy = abs (y2 - y1);
 
-		for (;y1 <= y2; y1++) {
-			gdImageSetPixel(im, x1,y1, color);
-		}
+	if (dx == 0) {
+		gdImageVLine(im, x1, y1, y2, color);
 		return;
-	} else if (y1==y2) { 	/* Horizontal */
-		if (x2 < x1) {
-			t = x2;
-			x2 = x1;
-			x1 = t;
-		}
-
-		for (;x1 <= x2; x1++) {
-			gdImageSetPixel(im, x1,y1, color);
-		}
+	} else if (dy == 0) {
+		gdImageHLine(im, y1, x1, x2, color);
 		return;
 	}
 
-	/* gdAntiAliased passed as color: set anti-aliased line (AAL) global vars. */
-	if (color == gdAntiAliased) {
-		im->AAL_x1 = x1;
-		im->AAL_y1 = y1;
-		im->AAL_x2 = x2;
-		im->AAL_y2 = y2;
-
-		/* Compute what we can for point-to-line distance calculation later. */
-		im->AAL_Bx_Ax = x2 - x1;
-		im->AAL_By_Ay = y2 - y1;
-		im->AAL_LAB_2 = (im->AAL_Bx_Ax * im->AAL_Bx_Ax) + (im->AAL_By_Ay * im->AAL_By_Ay);
-		im->AAL_LAB = sqrt (im->AAL_LAB_2);
-
-		/* For AA, we must draw pixels outside the width of the line.  Keep in
-		 * mind that this will be curtailed by cos/sin of theta later.
-		 */
-		thick += 4;
-	}
-
-	dx = abs(x2 - x1);
-	dy = abs(y2 - y1);
 	if (dy <= dx) {
 		/* More-or-less horizontal. use wid for vertical stroke */
 		/* Doug Claar: watch out for NaN in atan2 (2.0.5) */
 		if ((dx == 0) && (dy == 0)) {
 			wid = 1;
 		} else {
-			wid = (int)(thick * cos (atan2 (dy, dx)));
+			/* 2.0.12: Michael Schwartz: divide rather than multiply;
+TBB: but watch out for /0! */
+			double ac = cos (atan2 (dy, dx));
+			if (ac != 0) {
+				wid = thick / ac;
+			} else {
+				wid = 1;
+			}
 			if (wid == 0) {
 				wid = 1;
 			}
@@ -1140,15 +1183,16 @@ void gdImageLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
 	} else {
 		/* More-or-less vertical. use wid for horizontal stroke */
 		/* 2.0.12: Michael Schwartz: divide rather than multiply;
-		   TBB: but watch out for /0! */
-		double as = sin(atan2(dy, dx));
+           TBB: but watch out for /0! */
+		double as = sin (atan2 (dy, dx));
 		if (as != 0) {
-			if (!(wid = thick / as)) {
-				wid = 1;
-			}
+			wid = thick / as;
 		} else {
 			wid = 1;
 		}
+		if (wid == 0) {
+			wid = 1;
+		}
 
 		d = 2 * dx - dy;
 		incr1 = 2 * dx;
@@ -1201,11 +1245,6 @@ void gdImageLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
 			}
 		}
 	}
-
-	/* If this is the only line we are drawing, go ahead and blend. */
-	if (color == gdAntiAliased && !im->AA_polygon) {
-		gdImageAABlend(im);
-	}
 }
 
 
@@ -1631,13 +1670,30 @@ void gdImageFilledArc (gdImagePtr im, int cx, int cy, int w, int h, int s, int e
 	int lx = 0, ly = 0;
 	int fx = 0, fy = 0;
 
-	while (s<0) {
-		s += 360;
-	}
 
-	while (e < s) {
-		e += 360;
-	}
+    if ((s % 360)  == (e % 360)) {
+		s = 0; e = 360;
+    } else {
+        if (s > 360) {
+            s = s % 360;
+        }
+
+        if (e > 360) {
+            e = e % 360;
+        }
+
+        while (s < 0) {
+            s += 360;
+        }
+
+        while (e < s) {
+            e += 360;
+        }
+
+        if (s == e) {
+			s = 0; e = 360;
+        }
+    }
 
 	for (i = s; i <= e; i++) {
 		int x, y;
@@ -1890,7 +1946,7 @@ struct seg {int y, xl, xr, dy;};
 #define FILL_POP(Y, XL, XR, DY) \
     {sp--; Y = sp->y+(DY = sp->dy); XL = sp->xl; XR = sp->xr;}
 
-void _gdImageFillTiled(gdImagePtr im, int x, int y, int nc);
+static void _gdImageFillTiled(gdImagePtr im, int x, int y, int nc);
 
 void gdImageFill(gdImagePtr im, int x, int y, int nc)
 {
@@ -1992,16 +2048,16 @@ done:
 	im->alphaBlendingFlag = alphablending_bak;	
 }
 
-void _gdImageFillTiled(gdImagePtr im, int x, int y, int nc)
+static void _gdImageFillTiled(gdImagePtr im, int x, int y, int nc)
 {
-	int i,l, x1, x2, dy;
+	int l, x1, x2, dy;
 	int oc;   /* old pixel value */
 	int tiled;
 	int wx2,wy2;
 	/* stack of filled segments */
 	struct seg *stack;
 	struct seg *sp;
-	char **pts;
+	char *pts;
 
 	if (!im->tile) {
 		return;
@@ -2011,11 +2067,7 @@ void _gdImageFillTiled(gdImagePtr im, int x, int y, int nc)
 	tiled = nc==gdTiled;
 
 	nc =  gdImageTileGet(im,x,y);
-	pts = (char **) ecalloc(im->sy, sizeof(char*));
-
-	for (i=0; i<im->sy;i++) {
-		pts[i] = (char *) ecalloc(im->sx, sizeof(char));
-	}
+	pts = (char *) ecalloc(im->sy * im->sx, sizeof(char));
 
 	stack = (struct seg *)safe_emalloc(sizeof(struct seg), ((int)(im->sy*im->sx)/4), 1);
 	sp = stack;
@@ -2028,9 +2080,9 @@ void _gdImageFillTiled(gdImagePtr im, int x, int y, int nc)
  	FILL_PUSH(y+1, x, x, -1);
 	while (sp>stack) {
 		FILL_POP(y, x1, x2, dy);
-		for (x=x1; x>=0 && (!pts[y][x] && gdImageGetPixel(im,x,y)==oc); x--) {
+		for (x=x1; x>=0 && (!pts[y + x*wx2] && gdImageGetPixel(im,x,y)==oc); x--) {
 			nc = gdImageTileGet(im,x,y);
-			pts[y][x]=1;
+			pts[y + x*wx2]=1;
 			gdImageSetPixel(im,x, y, nc);
 		}
 		if (x>=x1) {
@@ -2044,9 +2096,9 @@ void _gdImageFillTiled(gdImagePtr im, int x, int y, int nc)
 		}
 		x = x1+1;
 		do {
-			for (; x<wx2 && (!pts[y][x] && gdImageGetPixel(im,x, y)==oc) ; x++) {
+			for (; x<wx2 && (!pts[y + x*wx2] && gdImageGetPixel(im,x, y)==oc) ; x++) {
 				nc = gdImageTileGet(im,x,y);
-				pts[y][x]=1;
+				pts[y + x*wx2]=1;
 				gdImageSetPixel(im, x, y, nc);
 			}
 			FILL_PUSH(y, l, x-1, dy);
@@ -2054,13 +2106,11 @@ void _gdImageFillTiled(gdImagePtr im, int x, int y, int nc)
 			if (x>x2+1) {
 				FILL_PUSH(y, x2+1, x-1, -dy);
 			}
-skip:			for (x++; x<=x2 && (pts[y][x] || gdImageGetPixel(im,x, y)!=oc); x++);
+skip:			for (x++; x<=x2 && (pts[y + x*wx2] || gdImageGetPixel(im,x, y)!=oc); x++);
 			l = x;
 		} while (x<=x2);
 	}
-	for (i=0; i<im->sy;i++) {
-		efree(pts[i]);
-	}
+
 	efree(pts);
 	efree(stack);
 }
@@ -2382,6 +2432,14 @@ void gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int
 	int *stx, *sty;
 	/* We only need to use floating point to determine the correct stretch vector for one line's worth. */
 	double accum;
+	
+	if (overflow2(sizeof(int), srcW)) {
+		return;
+	}
+	if (overflow2(sizeof(int), srcH)) {
+		return;
+	}
+
 	stx = (int *) gdMalloc (sizeof (int) * srcW);
 	sty = (int *) gdMalloc (sizeof (int) * srcH);
 	accum = 0;
@@ -2479,6 +2537,7 @@ void gdImageCopyResampled (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, i
 {
 	int x, y;
 	double sy1, sy2, sx1, sx2;
+
 	if (!dst->trueColor) {
 		gdImageCopyResized (dst, src, dstX, dstY, srcX, srcY, dstW, dstH, srcW, srcH);
 		return;
@@ -3173,6 +3232,10 @@ void gdImageFilledPolygon (gdImagePtr im, gdPointPtr p, int n, int c)
 		return;
 	}
 
+	if (overflow2(sizeof(int), n)) {
+		return;
+	}
+
 	if (c == gdAntiAliased) {
 		fill_color = im->AA_color;
 	} else {
@@ -3187,6 +3250,9 @@ void gdImageFilledPolygon (gdImagePtr im, gdPointPtr p, int n, int c)
 		while (im->polyAllocated < n) {
 			im->polyAllocated *= 2;
 		}
+		if (overflow2(sizeof(int), im->polyAllocated)) {
+			return;
+		}
 		im->polyInts = (int *) gdRealloc(im->polyInts, sizeof(int) * im->polyAllocated);
 	}
 	miny = p[0].y;
@@ -3398,7 +3464,7 @@ int gdImageCompare (gdImagePtr im1, gdImagePtr im2)
 }
 
 int
-gdAlphaBlend (int dst, int src)
+gdAlphaBlendOld (int dst, int src)
 {
 	/* 2.0.12: TBB: alpha in the destination should be a
 	 * component of the result. Thanks to Frank Warmerdam for
@@ -3420,6 +3486,51 @@ gdAlphaBlend (int dst, int src)
 	    gdTrueColorGetBlue (dst)) / gdAlphaMax));
 }
 
+int gdAlphaBlend (int dst, int src) {
+    int src_alpha = gdTrueColorGetAlpha(src);
+    int dst_alpha, alpha, red, green, blue;
+    int src_weight, dst_weight, tot_weight;
+
+/* -------------------------------------------------------------------- */
+/*      Simple cases we want to handle fast.                            */
+/* -------------------------------------------------------------------- */
+    if( src_alpha == gdAlphaOpaque )
+        return src;
+
+    dst_alpha = gdTrueColorGetAlpha(dst);
+    if( src_alpha == gdAlphaTransparent )
+        return dst;
+    if( dst_alpha == gdAlphaTransparent )
+        return src;
+
+/* -------------------------------------------------------------------- */
+/*      What will the source and destination alphas be?  Note that      */
+/*      the destination weighting is substantially reduced as the       */
+/*      overlay becomes quite opaque.                                   */
+/* -------------------------------------------------------------------- */
+    src_weight = gdAlphaTransparent - src_alpha;
+    dst_weight = (gdAlphaTransparent - dst_alpha) * src_alpha / gdAlphaMax;
+    tot_weight = src_weight + dst_weight;
+    
+/* -------------------------------------------------------------------- */
+/*      What red, green and blue result values will we use?             */
+/* -------------------------------------------------------------------- */
+    alpha = src_alpha * dst_alpha / gdAlphaMax;
+
+    red = (gdTrueColorGetRed(src) * src_weight
+           + gdTrueColorGetRed(dst) * dst_weight) / tot_weight;
+    green = (gdTrueColorGetGreen(src) * src_weight
+           + gdTrueColorGetGreen(dst) * dst_weight) / tot_weight;
+    blue = (gdTrueColorGetBlue(src) * src_weight
+           + gdTrueColorGetBlue(dst) * dst_weight) / tot_weight;
+
+/* -------------------------------------------------------------------- */
+/*      Return merged result.                                           */
+/* -------------------------------------------------------------------- */
+    return ((alpha << 24) + (red << 16) + (green << 8) + blue);
+
+}
+
 void gdImageAlphaBlending (gdImagePtr im, int alphaBlendingArg)
 {
 	im->alphaBlendingFlag = alphaBlendingArg;
@@ -3546,9 +3657,7 @@ int gdImageNegate(gdImagePtr src)
 			if (new_pxl == -1) {
 				new_pxl = gdImageColorClosestAlpha(src, 255-r, 255-g, 255-b, a);
 			}
-			if ((y >= 0) && (y < src->sy)) {
-				gdImageSetPixel (src, x, y, new_pxl);
-			}
+			gdImageSetPixel (src, x, y, new_pxl);
 		}
 	}
 	return 1;
@@ -3581,9 +3690,7 @@ int gdImageGrayScale(gdImagePtr src)
 			if (new_pxl == -1) {
 				new_pxl = gdImageColorClosestAlpha(src, r, g, b, a);
 			}
-			if ((y >= 0) && (y < src->sy)) {
-				gdImageSetPixel (src, x, y, new_pxl);
-			}
+			gdImageSetPixel (src, x, y, new_pxl);
 		}
 	}
 	return 1;
@@ -3628,9 +3735,7 @@ int gdImageBrightness(gdImagePtr src, int brightness)
 			if (new_pxl == -1) {
 				new_pxl = gdImageColorClosestAlpha(src, (int)r, (int)g, (int)b, a);
 			}
-			if ((y >= 0) && (y < src->sy)) {
-				gdImageSetPixel (src, x, y, new_pxl);
-			}
+			gdImageSetPixel (src, x, y, new_pxl);
 		}
 	}
 	return 1;
@@ -3690,9 +3795,7 @@ int gdImageContrast(gdImagePtr src, double contrast)
 			if (new_pxl == -1) {
 				new_pxl = gdImageColorClosestAlpha(src, (int)rf, (int)gf, (int)bf, a);
 			}
-			if ((y >= 0) && (y < src->sy)) {
-				gdImageSetPixel (src, x, y, new_pxl);
-			}
+			gdImageSetPixel (src, x, y, new_pxl);
 		}
 	}
 	return 1;
@@ -3733,9 +3836,7 @@ int gdImageColor(gdImagePtr src, int red, int green, int blue)
 			if (new_pxl == -1) {
 				new_pxl = gdImageColorClosestAlpha(src, (int)r, (int)g, (int)b, a);
 			}
-			if ((y >= 0) && (y < src->sy)) {
-				gdImageSetPixel (src, x, y, new_pxl);
-			}
+			gdImageSetPixel (src, x, y, new_pxl);
 		}
 	}
 	return 1;
@@ -3791,9 +3892,7 @@ int gdImageConvolution(gdImagePtr src, float filter[3][3], float filter_div, flo
 			if (new_pxl == -1) {
 				new_pxl = gdImageColorClosestAlpha(src, (int)new_r, (int)new_g, (int)new_b, new_a);
 			}
-			if ((y >= 0) && (y < src->sy)) {
-				gdImageSetPixel (src, x, y, new_pxl);
-			}
+			gdImageSetPixel (src, x, y, new_pxl);
 		}
 	}
 	gdImageDestroy(srcback);
@@ -3913,9 +4012,7 @@ int gdImageSelectiveBlur( gdImagePtr src)
 			if (new_pxl == -1) {
 				new_pxl = gdImageColorClosestAlpha(src, (int)new_r, (int)new_g, (int)new_b, new_a);
 			}
-			if ((y >= 0) && (y < src->sy)) {
-				gdImageSetPixel (src, x, y, new_pxl);
-			}
+			gdImageSetPixel (src, x, y, new_pxl);
 		}
 	}
 	gdImageDestroy(srcback);
