@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: json.c,v 1.9.2.11 2007/01/12 12:17:32 tony2001 Exp $ */
+/* $Id: json.c,v 1.9.2.19 2007/07/24 22:57:13 bjori Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -86,7 +86,8 @@ static PHP_MINFO_FUNCTION(json)
 static void json_encode_r(smart_str *buf, zval *val TSRMLS_DC);
 static void json_escape_string(smart_str *buf, char *s, int len);
 
-static int json_determine_array_type(zval **val TSRMLS_DC) {
+static int json_determine_array_type(zval **val TSRMLS_DC)  /* {{{ */
+{
     int i;
     HashTable *myht = HASH_OF(*val);
 
@@ -117,8 +118,9 @@ static int json_determine_array_type(zval **val TSRMLS_DC) {
 
     return 0;
 }
+/* }}} */
 
-static void json_encode_array(smart_str *buf, zval **val TSRMLS_DC) {
+static void json_encode_array(smart_str *buf, zval **val TSRMLS_DC) { /* {{{ */
     int i, r;
     HashTable *myht;
 
@@ -177,7 +179,7 @@ static void json_encode_array(smart_str *buf, zval **val TSRMLS_DC) {
                     json_encode_r(buf, *data TSRMLS_CC);
                 } else if (r == 1) {
                     if (i == HASH_KEY_IS_STRING) {
-                        if (key[0] == '\0') {
+                        if (key[0] == '\0' && Z_TYPE_PP(val) == IS_OBJECT) {
                             /* Skip protected and private members. */
                             continue;
                         }
@@ -224,10 +226,11 @@ static void json_encode_array(smart_str *buf, zval **val TSRMLS_DC) {
         smart_str_appendc(buf, '}');
     }
 }
+/* }}} */
 
 #define REVERSE16(us) (((us & 0xf) << 12) | (((us >> 4) & 0xf) << 8) | (((us >> 8) & 0xf) << 4) | ((us >> 12) & 0xf))
 
-static void json_escape_string(smart_str *buf, char *s, int len)
+static void json_escape_string(smart_str *buf, char *s, int len) /* {{{ */
 {
     int pos = 0;
     unsigned short us;
@@ -303,7 +306,7 @@ static void json_escape_string(smart_str *buf, char *s, int len)
                 break;
             default:
                 {
-                    if (us < ' ' || (us & 127) == us)
+                    if (us >= ' ' && (us & 127) == us)
                     {
                         smart_str_appendc(buf, (unsigned char) us);
                     }
@@ -328,8 +331,10 @@ static void json_escape_string(smart_str *buf, char *s, int len)
     smart_str_appendc(buf, '"');
     efree(utf16);
 }
+/* }}} */
 
-static void json_encode_r(smart_str *buf, zval *val TSRMLS_DC) {
+static void json_encode_r(smart_str *buf, zval *val TSRMLS_DC) /* {{{ */
+{
     switch (Z_TYPE_P(val)) {
         case IS_NULL:
             smart_str_appendl(buf, "null", 4);
@@ -345,7 +350,7 @@ static void json_encode_r(smart_str *buf, zval *val TSRMLS_DC) {
             }
             break;
         case IS_LONG:
-            smart_str_append_long(buf, Z_LVAL_P(val));
+	    smart_str_append_long(buf, Z_LVAL_P(val));
             break;
         case IS_DOUBLE:
             {
@@ -353,17 +358,11 @@ static void json_encode_r(smart_str *buf, zval *val TSRMLS_DC) {
                 int len;
                 double dbl = Z_DVAL_P(val);
 
-                if (!zend_isinf(dbl) && !zend_isnan(dbl))
-                {
-                    len = spprintf(&d, 0, "%.9g", dbl);
-                    if (d)
-                    {
-                        smart_str_appendl(buf, d, len);
-                        efree(d);
-                    }
-                }
-                else
-                {
+                if (!zend_isinf(dbl) && !zend_isnan(dbl)) {
+			len = spprintf(&d, 0, "%.*g", (int) EG(precision), dbl);
+			smart_str_appendl(buf, d, len);
+			efree(d);
+                } else {
                     zend_error(E_WARNING, "[json] (json_encode_r) double %.9g does not conform to the JSON spec, encoded as 0.", dbl);
                     smart_str_appendc(buf, '0');
                 }
@@ -384,7 +383,10 @@ static void json_encode_r(smart_str *buf, zval *val TSRMLS_DC) {
 
     return;
 }
+/* }}} */
 
+/* {{{ proto string json_encode(mixed data)
+   Returns the JSON representation of a value */
 static PHP_FUNCTION(json_encode)
 {
     zval *parameter;
@@ -400,7 +402,10 @@ static PHP_FUNCTION(json_encode)
 
     smart_str_free(&buf);
 }
+/* }}} */
 
+/* {{{ proto mixed json_decode(string json [, bool assoc])
+   Decodes the JSON representation into a PHP value */
 static PHP_FUNCTION(json_decode)
 {
     char *parameter;
@@ -465,7 +470,7 @@ static PHP_FUNCTION(json_decode)
 			RETURN_DOUBLE(d);
 		}
 	}
-	if (*parameter == '"' && parameter[parameter_len-1] == '"') {
+	if (parameter_len > 1 && *parameter == '"' && parameter[parameter_len-1] == '"') {
 	        RETURN_STRINGL(parameter+1, parameter_len-2, 1);
 	} else if (*parameter == '{' || *parameter == '[') { /* invalid JSON string */
 		RETURN_NULL();
@@ -474,6 +479,7 @@ static PHP_FUNCTION(json_decode)
 	}
     }
 }
+/* }}} */
 
 /*
  * Local variables:
