@@ -15,7 +15,7 @@
   | Author: Georg Richter <georg@php.net>                                |
   +----------------------------------------------------------------------+
 
-  $Id: mysqli.c,v 1.72.2.16.2.12 2007/01/01 09:36:03 sebastian Exp $ 
+  $Id: mysqli.c,v 1.72.2.16.2.16 2007/06/25 16:01:30 scottmac Exp $ 
 */
 
 #ifdef HAVE_CONFIG_H
@@ -415,8 +415,21 @@ PHP_MYSQLI_EXPORT(zend_object_value) mysqli_objects_new(zend_class_entry *class_
 	
 /* {{{ mysqli_module_entry
  */
+/* Dependancies */
+static zend_module_dep mysqli_deps[] = {
+#if defined(HAVE_SPL) && ((PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1))
+	ZEND_MOD_REQUIRED("spl")
+#endif
+	{NULL, NULL, NULL}
+};
+
 zend_module_entry mysqli_module_entry = {
+#if ZEND_MODULE_API_NO >= 20050922
+	STANDARD_MODULE_HEADER_EX, NULL,
+	mysqli_deps,
+#elif ZEND_MODULE_API_NO >= 20010901
 	STANDARD_MODULE_HEADER,
+#endif
 	"mysqli",
 	mysqli_functions,
 	PHP_MINIT(mysqli),
@@ -639,6 +652,10 @@ PHP_MINIT_FUNCTION(mysqli)
 	REGISTER_LONG_CONSTANT("MYSQLI_REPORT_ALL", MYSQLI_REPORT_ALL, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("MYSQLI_REPORT_OFF", 0, CONST_CS | CONST_PERSISTENT);
 
+	if (mysql_server_init(0, NULL, NULL)) {
+		return FAILURE;
+	}
+
 	return SUCCESS;
 }
 /* }}} */
@@ -647,6 +664,16 @@ PHP_MINIT_FUNCTION(mysqli)
  */
 PHP_MSHUTDOWN_FUNCTION(mysqli)
 {
+#ifdef PHP_WIN32
+	unsigned long client_ver = mysql_get_client_version;
+	/* Can't call mysql_server_end() multiple times prior to 5.0.42 on Windows */
+	if ((client_ver > 50042 && client_ver < 50100) || client_ver > 50122) {
+		mysql_server_end();
+	}
+#else
+	mysql_server_end();
+#endif
+
 	zend_hash_destroy(&mysqli_driver_properties);
 	zend_hash_destroy(&mysqli_result_properties);
 	zend_hash_destroy(&mysqli_stmt_properties);
@@ -663,6 +690,11 @@ PHP_MSHUTDOWN_FUNCTION(mysqli)
  */
 PHP_RINIT_FUNCTION(mysqli)
 {
+#ifdef ZTS
+	if (mysql_thread_init()) {
+		return FAILURE;
+	}
+#endif
 	MyG(error_msg) = NULL;
 	MyG(error_no) = 0;
 
@@ -674,6 +706,9 @@ PHP_RINIT_FUNCTION(mysqli)
  */
 PHP_RSHUTDOWN_FUNCTION(mysqli)
 {
+#ifdef ZTS
+	mysql_thread_end();
+#endif
 	if (MyG(error_msg)) {
 		efree(MyG(error_msg));
 	}
@@ -1037,7 +1072,7 @@ int php_local_infile_init(void **ptr, const char *filename, void *userdata)
 	mysql->li_stream = php_stream_open_wrapper_ex((char *)filename, "r", 0, NULL, context);
 
 	if (mysql->li_stream == NULL) {
-		sprintf((char *)data->error_msg, "Can't find file '%-.64s'.", filename);	
+		snprintf((char *)data->error_msg, sizeof(data->error_msg), "Can't find file '%-.64s'.", filename);	
 		return 1;
 	}
 
@@ -1130,10 +1165,10 @@ int php_local_infile_error(void *ptr, char *error_msg, uint error_msg_len)
 	mysqli_local_infile *data = (mysqli_local_infile *) ptr;
 
 	if (data) {
-		strcpy(error_msg, data->error_msg);
+		strlcpy(error_msg, data->error_msg, error_msg_len);
 		return 2000;
 	} 
-	strcpy(error_msg, ER(CR_OUT_OF_MEMORY));
+	strlcpy(error_msg, ER(CR_OUT_OF_MEMORY), error_msg_len);
 	return CR_OUT_OF_MEMORY;
 }
 /* }}} */
