@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2007 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: oci8_interface.c,v 1.8.2.7.2.3 2006/08/22 11:09:12 tony2001 Exp $ */
+/* $Id: oci8_interface.c,v 1.8.2.7.2.13 2007/08/02 19:04:37 sixd Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -40,6 +40,10 @@
 #include "php_oci8.h"
 #include "php_oci8_int.h"
 
+#ifndef OCI_STMT_CALL
+#define OCI_STMT_CALL 10
+#endif
+
 /* {{{ proto bool oci_define_by_name(resource stmt, string name, mixed &var [, int type])
    Define a PHP variable to an Oracle column by name */
 /* if you want to define a LOB/CLOB etc make sure you allocate it via OCINewDescriptor BEFORE defining!!! */
@@ -48,7 +52,7 @@ PHP_FUNCTION(oci_define_by_name)
 	zval *stmt, *var;
 	char *name;
 	int name_len;
-	long type = SQLT_CHR;
+	long type = 0;
 	php_oci_statement *statement;
 	php_oci_define *define, *tmp_define;
 
@@ -587,7 +591,8 @@ PHP_FUNCTION(oci_lob_truncate)
 {
 	zval **tmp, *z_descriptor = getThis();
 	php_oci_descriptor *descriptor;
-	ub4 trim_length = 0;
+	long trim_length = 0;
+	ub4 ub_trim_length;
 	
 	if (getThis()) {
 		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &trim_length) == FAILURE) {
@@ -604,10 +609,16 @@ PHP_FUNCTION(oci_lob_truncate)
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to find descriptor property");
 		RETURN_FALSE;
 	}
-	
+
+	if (trim_length < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length must be greater than or equal to zero");
+		RETURN_FALSE;
+	}
+
+	ub_trim_length = (ub4) trim_length;
 	PHP_OCI_ZVAL_TO_DESCRIPTOR(*tmp, descriptor);
 	
-	if (php_oci_lob_truncate(descriptor, trim_length TSRMLS_CC)) {
+	if (php_oci_lob_truncate(descriptor, ub_trim_length TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 	RETURN_TRUE;
@@ -674,7 +685,7 @@ PHP_FUNCTION(oci_lob_flush)
 {
 	zval **tmp, *z_descriptor = getThis();
 	php_oci_descriptor *descriptor;
-	ub4 flush_flag = 0;
+	long flush_flag = 0;
 	
 	if (getThis()) {
 		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &flush_flag) == FAILURE) {
@@ -1172,7 +1183,22 @@ PHP_FUNCTION(oci_field_type)
 #endif
 #ifdef SQLT_TIMESTAMP_TZ
 		case SQLT_TIMESTAMP_TZ:
-			RETVAL_STRING("TIMESTAMP_TZ",1);
+			RETVAL_STRING("TIMESTAMP WITH TIMEZONE",1);
+			break;
+#endif
+#ifdef SQLT_TIMESTAMP_LTZ
+		case SQLT_TIMESTAMP_LTZ:
+			RETVAL_STRING("TIMESTAMP WITH LOCAL TIMEZONE",1);
+			break;
+#endif
+#ifdef SQLT_INTERVAL_YM
+		case SQLT_INTERVAL_YM:
+			RETVAL_STRING("INTERVAL YEAR TO MONTH",1);
+			break;
+#endif
+#ifdef SQLT_INTERVAL_DS
+		case SQLT_INTERVAL_DS:
+			RETVAL_STRING("INTERVAL DAY TO SECOND",1);
 			break;
 #endif
 		case SQLT_DAT:
@@ -1549,7 +1575,7 @@ PHP_FUNCTION(oci_error)
 	php_oci_connection *connection;
 	text *errbuf;
 	sb4 errcode = 0;
-	sword error = 0;
+	sword error = OCI_SUCCESS;
 	dvoid *errh = NULL;
 #ifdef HAVE_OCI8_ATTR_STATEMENT
 	ub2 error_offset = 0;
@@ -1572,19 +1598,28 @@ PHP_FUNCTION(oci_error)
 				RETURN_FALSE;
 			}
 #endif
-		} else {
-			connection = (php_oci_connection *) zend_fetch_resource(&arg TSRMLS_CC, -1, NULL, NULL, 1, le_connection);
-			
-			if (connection) {
-				errh = connection->err;
-				error = connection->errcode;
-			}
+			goto go_out;
+		}
+
+		connection = (php_oci_connection *) zend_fetch_resource(&arg TSRMLS_CC, -1, NULL, NULL, 1, le_connection);
+		if (connection) {
+			errh = connection->err;
+			error = connection->errcode;
+			goto go_out;
+		}
+
+		connection = (php_oci_connection *) zend_fetch_resource(&arg TSRMLS_CC, -1, NULL, NULL, 1, le_pconnection);
+		if (connection) {
+			errh = connection->err;
+			error = connection->errcode;
+			goto go_out;
 		}
 	} else {
 		errh = OCI_G(err);
 		error = OCI_G(errcode);
 	}
 
+go_out:
 	if (error == OCI_SUCCESS) { /* no error set in the handle */
 		RETURN_FALSE;
 	}
@@ -1704,7 +1739,7 @@ PHP_FUNCTION(oci_password_change)
 			RETURN_FALSE;
 		}
 
-		if (php_oci_password_change(connection, user, user_len, pass_old, pass_old_len, pass_new, pass_new_len TSRMLS_CC)) {
+		if (php_oci_password_change(connection, (char *)user, user_len, (char *)pass_old, pass_old_len, (char *)pass_new, pass_new_len TSRMLS_CC)) {
 			RETURN_FALSE;
 		}
 		RETURN_TRUE;
@@ -1723,7 +1758,7 @@ PHP_FUNCTION(oci_password_change)
 			RETURN_FALSE;
 		}
 
-		connection = php_oci_do_connect_ex(user, user_len, pass_old, pass_old_len, pass_new, pass_new_len, dbname, dbname_len, NULL, OCI_DEFAULT, 0, 0 TSRMLS_CC);
+		connection = php_oci_do_connect_ex((char *)user, user_len, (char *)pass_old, pass_old_len, (char *)pass_new, pass_new_len, (char *)dbname, dbname_len, NULL, OCI_DEFAULT, 0, 0 TSRMLS_CC);
 		if (!connection) {
 			RETURN_FALSE;
 		}
@@ -1840,6 +1875,9 @@ PHP_FUNCTION(oci_statement_type)
 		case OCI_STMT_DECLARE:
 			RETVAL_STRING("DECLARE",1);
 			break;
+		case OCI_STMT_CALL:
+			RETVAL_STRING("CALL",1);
+			break;
 		default:
 			RETVAL_STRING("UNKNOWN",1);
 	}
@@ -2155,3 +2193,12 @@ PHP_FUNCTION(oci_new_collection)
 #endif
 
 #endif /* HAVE_OCI8 */
+
+/*
+ * Local variables:
+ * tab-width: 4
+ * c-basic-offset: 4
+ * End:
+ * vim600: noet sw=4 ts=4 fdm=marker
+ * vim<600: noet sw=4 ts=4
+ */
