@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2009 The PHP Group                                |
+   | Copyright (c) 1997-2007 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
  
-/* $Id: php_pcre.h,v 1.41.2.1.2.5.2.3 2008/12/31 11:15:40 sebastian Exp $ */
+/* $Id: php_pcre.h,v 1.41.2.1.2.5 2007/01/01 09:36:04 sebastian Exp $ */
 
 #ifndef PHP_PCRE_H
 #define PHP_PCRE_H
@@ -33,6 +33,14 @@
 #include <locale.h>
 #endif
 
+PHP_FUNCTION(preg_match);
+PHP_FUNCTION(preg_match_all);
+PHP_FUNCTION(preg_replace);
+PHP_FUNCTION(preg_replace_callback);
+PHP_FUNCTION(preg_split);
+PHP_FUNCTION(preg_quote);
+PHP_FUNCTION(preg_grep);
+
 PHPAPI char *php_pcre_replace(char *regex, int regex_len, char *subject, int subject_len, zval *replace_val, int is_callable_replace, int *result_len, int limit, int *replace_count TSRMLS_DC);
 PHPAPI pcre* pcre_get_compiled_regex(char *regex, pcre_extra **extra, int *options TSRMLS_DC);
 PHPAPI pcre* pcre_get_compiled_regex_ex(char *regex, pcre_extra **extra, int *preg_options, int *coptions TSRMLS_DC);
