@@ -18,7 +18,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: pdo_stmt.c,v 1.118.2.38.2.14 2007/01/27 21:53:26 tony2001 Exp $ */
+/* $Id: pdo_stmt.c,v 1.118.2.38.2.22 2007/07/31 22:48:42 iliaa Exp $ */
 
 /* The PDO Statement Handle Class */
 
@@ -251,7 +251,10 @@ static void param_dtor(void *data) /* {{{ */
 		efree(param->name);
 	}
 
-	zval_ptr_dtor(&(param->parameter));
+	if (param->parameter) {
+		zval_ptr_dtor(&(param->parameter));
+		param->parameter = NULL;
+	}
 	if (param->driver_params) {
 		zval_ptr_dtor(&(param->driver_params));
 	}
@@ -277,7 +280,13 @@ static int really_register_bound_param(struct pdo_bound_param_data *param, pdo_s
 	}
 
 	if (PDO_PARAM_TYPE(param->param_type) == PDO_PARAM_STR && param->max_value_len <= 0 && ! ZVAL_IS_NULL(param->parameter)) {
-		convert_to_string(param->parameter);
+		if (Z_TYPE_P(param->parameter) == IS_DOUBLE) {
+			char *p;
+			int len = spprintf(&p, 0, "%F", Z_DVAL_P(param->parameter));
+			ZVAL_STRINGL(param->parameter, p, len, 0);
+		} else {
+			convert_to_string(param->parameter);
+		}
 	} else if (PDO_PARAM_TYPE(param->param_type) == PDO_PARAM_INT && Z_TYPE_P(param->parameter) == IS_BOOL) {
 		convert_to_long(param->parameter);
 	} else if (PDO_PARAM_TYPE(param->param_type) == PDO_PARAM_BOOL && Z_TYPE_P(param->parameter) == IS_LONG) {
@@ -858,10 +867,10 @@ static int do_fetch(pdo_stmt_t *stmt, int do_bind, zval *return_value,
 	zend_class_entry *ce = NULL, *old_ce = NULL;
 	zval grp_val, *grp, **pgrp, *retval, *old_ctor_args = NULL;
 
-	how = how & ~PDO_FETCH_FLAGS;
 	if (how == PDO_FETCH_USE_DEFAULT) {
 		how = stmt->default_fetch_type;
 	}
+	how = how & ~PDO_FETCH_FLAGS;
 
 	if (!do_fetch_common(stmt, ori, offset, do_bind TSRMLS_CC)) {
 		return 0;
@@ -897,6 +906,16 @@ static int do_fetch(pdo_stmt_t *stmt, int do_bind, zval *return_value,
 				}
 				break;
 
+			case PDO_FETCH_KEY_PAIR:
+				if (stmt->column_count != 2) {
+					pdo_raise_impl_error(stmt->dbh, stmt, "HY000", "PDO::FETCH_KEY_PAIR fetch mode requires the result set to contain extactly 2 columns." TSRMLS_CC);
+					return 0;
+				}
+				if (!return_all) {
+					array_init(return_value);
+				}
+				break;
+
 			case PDO_FETCH_COLUMN:
 				if (stmt->fetch.column >= 0 && stmt->fetch.column < stmt->column_count) {
 					fetch_value(stmt, return_value, stmt->fetch.column, NULL TSRMLS_CC);
@@ -1000,7 +1019,7 @@ static int do_fetch(pdo_stmt_t *stmt, int do_bind, zval *return_value,
 				return 0;
 		}
 		
-		if (return_all) {
+		if (return_all && how != PDO_FETCH_KEY_PAIR) {
 			INIT_PZVAL(&grp_val);
 			fetch_value(stmt, &grp_val, i, NULL TSRMLS_CC);
 			convert_to_string(&grp_val);
@@ -1020,6 +1039,23 @@ static int do_fetch(pdo_stmt_t *stmt, int do_bind, zval *return_value,
 				case PDO_FETCH_ASSOC:
 					add_assoc_zval(return_value, stmt->columns[i].name, val);
 					break;
+					
+				case PDO_FETCH_KEY_PAIR:
+					{
+						zval *tmp;
+						MAKE_STD_ZVAL(tmp);
+						fetch_value(stmt, tmp, ++i, NULL TSRMLS_CC);
+
+						if (Z_TYPE_P(val) == IS_LONG) {
+							zend_hash_index_update((return_all ? Z_ARRVAL_P(return_all) : Z_ARRVAL_P(return_value)), Z_LVAL_P(val), &tmp, sizeof(zval *), NULL);
+						} else {
+							convert_to_string(val);
+							zend_symtable_update((return_all ? Z_ARRVAL_P(return_all) : Z_ARRVAL_P(return_value)), Z_STRVAL_P(val), Z_STRLEN_P(val) + 1, &tmp, sizeof(zval *), NULL);
+						}
+						zval_ptr_dtor(&val);
+						return 1;
+					}
+					break;
 
 				case PDO_FETCH_USE_DEFAULT:
 				case PDO_FETCH_BOTH:
@@ -1485,7 +1521,7 @@ static PHP_METHOD(PDOStatement, fetchAll)
 	if (!error)	{
 		PDO_STMT_CLEAR_ERR();
 		MAKE_STD_ZVAL(data);
-		if (how & PDO_FETCH_GROUP) {
+		if ((how & PDO_FETCH_GROUP) || how == PDO_FETCH_KEY_PAIR) {
 			array_init(return_value);
 			return_all = return_value;
 		} else {
@@ -1501,6 +1537,8 @@ static PHP_METHOD(PDOStatement, fetchAll)
 			do {
 				MAKE_STD_ZVAL(data);
 			} while (do_fetch(stmt, TRUE, data, how, PDO_FETCH_ORI_NEXT, 0, return_all TSRMLS_CC));
+		} else if (how == PDO_FETCH_KEY_PAIR) {
+			while (do_fetch(stmt, TRUE, data, how, PDO_FETCH_ORI_NEXT, 0, return_all TSRMLS_CC));
 		} else {
 			array_init(return_value);
 			do {
@@ -1557,7 +1595,10 @@ static int register_bound_param(INTERNAL_FUNCTION_PARAMETERS, pdo_stmt_t *stmt,
 
 	ZVAL_ADDREF(param.parameter);
 	if (!really_register_bound_param(&param, stmt, is_param TSRMLS_CC)) {
-		zval_ptr_dtor(&(param.parameter));
+		if (param.parameter) {
+			zval_ptr_dtor(&(param.parameter));
+			param.parameter = NULL;
+		}
 		return 0;
 	}
 	return 1;
@@ -1590,7 +1631,10 @@ static PHP_METHOD(PDOStatement, bindValue)
 	
 	ZVAL_ADDREF(param.parameter);
 	if (!really_register_bound_param(&param, stmt, TRUE TSRMLS_CC)) {
-		zval_ptr_dtor(&(param.parameter));
+		if (param.parameter) {
+			zval_ptr_dtor(&(param.parameter));
+			param.parameter = NULL;
+		}
 		RETURN_FALSE;
 	}
 	RETURN_TRUE;
@@ -2516,28 +2560,18 @@ static void row_prop_or_dim_delete(zval *object, zval *offset TSRMLS_DC)
 
 static HashTable *row_get_properties(zval *object TSRMLS_DC)
 {
-	zval *tmp;
 	pdo_stmt_t * stmt = (pdo_stmt_t *) zend_object_store_get_object(object TSRMLS_CC);
 	int i;
-	HashTable *ht;
-
-	MAKE_STD_ZVAL(tmp);
-	array_init(tmp);
 
 	for (i = 0; i < stmt->column_count; i++) {
 		zval *val;
 		MAKE_STD_ZVAL(val);
 		fetch_value(stmt, val, i, NULL TSRMLS_CC);
 
-		add_assoc_zval(tmp, stmt->columns[i].name, val);
+		zend_hash_update(stmt->properties, stmt->columns[i].name, stmt->columns[i].namelen + 1, (void *)&val, sizeof(zval *), NULL);
 	}
 
-	ht = Z_ARRVAL_P(tmp);
-
-	ZVAL_NULL(tmp);
-	FREE_ZVAL(tmp);
-
-	return ht;
+	return stmt->properties;
 }
 
 static union _zend_function *row_method_get(
