@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2006 The PHP Group                                |
+  | Copyright (c) 1997-2007 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: oci_statement.c,v 1.16.2.10 2006/03/18 22:06:30 tony2001 Exp $ */
+/* $Id: oci_statement.c,v 1.16.2.10.2.6 2007/06/30 02:30:34 sixd Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -32,18 +32,22 @@
 #include "Zend/zend_extensions.h"
 
 #define STMT_CALL(name, params)	\
-	S->last_err = name params; \
-	S->last_err = _oci_error(S->err, stmt->dbh, stmt, #name, S->last_err, __FILE__, __LINE__ TSRMLS_CC); \
-	if (S->last_err) { \
-		return 0; \
-	}
+	do { \
+		S->last_err = name params;										\
+		S->last_err = _oci_error(S->err, stmt->dbh, stmt, #name, S->last_err, FALSE, __FILE__, __LINE__ TSRMLS_CC); \
+		if (S->last_err) {												\
+			return 0;													\
+		}																\
+	} while(0)
 
 #define STMT_CALL_MSG(name, msg, params)	\
-	S->last_err = name params; \
-	S->last_err = _oci_error(S->err, stmt->dbh, stmt, #name ": " #msg, S->last_err, __FILE__, __LINE__ TSRMLS_CC); \
-	if (S->last_err) { \
-		return 0; \
-	}
+	do { \
+		S->last_err = name params;										\
+		S->last_err = _oci_error(S->err, stmt->dbh, stmt, #name ": " #msg, S->last_err, FALSE, __FILE__, __LINE__ TSRMLS_CC); \
+		if (S->last_err) {												\
+			return 0;													\
+		}																\
+	} while(0)
 
 static php_stream *oci_create_lob_stream(pdo_stmt_t *stmt, OCILobLocator *lob TSRMLS_DC);
 
@@ -52,7 +56,7 @@ static int oci_stmt_dtor(pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 	pdo_oci_stmt *S = (pdo_oci_stmt*)stmt->driver_data;
 	HashTable *BC = stmt->bound_columns;
 	HashTable *BP = stmt->bound_params;
-	
+
 	int i;
 
 	if (S->stmt) {
@@ -75,18 +79,18 @@ static int oci_stmt_dtor(pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 		FREE_HASHTABLE(stmt->bound_columns);
 		stmt->bound_columns = NULL;
 	}
-	
+
 	if (BP) {
 		zend_hash_destroy(BP);
 		FREE_HASHTABLE(stmt->bound_params);
 		stmt->bound_params = NULL;
 	}
-	
+
 	if (S->einfo.errmsg) {
 		efree(S->einfo.errmsg);
 		S->einfo.errmsg = NULL;
 	}
-	
+
 	if (S->cols) {
 		for (i = 0; i < stmt->column_count; i++) {
 			if (S->cols[i].data) {
@@ -151,9 +155,26 @@ static int oci_stmt_execute(pdo_stmt_t *stmt TSRMLS_DC) /* {{{ */
 
 		stmt->column_count = (int)colcount;
 
+		if (S->cols) {
+			int i;
+			for (i = 0; i < stmt->column_count; i++) {
+				if (S->cols[i].data) {
+					switch (S->cols[i].dtype) {
+						case SQLT_BLOB:
+						case SQLT_CLOB:
+							/* do nothing */
+							break;
+						default:
+							efree(S->cols[i].data);
+					}
+				}
+			}
+			efree(S->cols);
+		}
+
 		S->cols = ecalloc(colcount, sizeof(pdo_oci_column));
 	}
-	
+
 	STMT_CALL_MSG(OCIAttrGet, "ATTR_ROW_COUNT",
 			(S->stmt, OCI_HTYPE_STMT, &rowcount, 0, OCI_ATTR_ROW_COUNT, S->err));
 	stmt->row_count = (long)rowcount;
@@ -171,7 +192,7 @@ static sb4 oci_bind_input_cb(dvoid *ctx, OCIBind *bindp, ub4 iter, ub4 index, dv
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "param is NULL in oci_bind_input_cb; this should not happen");
 		return OCI_ERROR;
 	}
-	
+
 	*indpp = &P->indicator;
 
 	if (P->thing) {
@@ -212,8 +233,8 @@ static sb4 oci_bind_output_cb(dvoid *ctx, OCIBind *bindp, ub4 iter, ub4 index, d
 		*rcodepp = &P->retcode;
 		*indpp = &P->indicator;
 		return OCI_CONTINUE;
-	} 
-	
+	}
+
 	if (Z_TYPE_P(param->parameter) == IS_OBJECT || Z_TYPE_P(param->parameter) == IS_RESOURCE) {
 		return OCI_CONTINUE;
 	}
@@ -222,10 +243,10 @@ static sb4 oci_bind_output_cb(dvoid *ctx, OCIBind *bindp, ub4 iter, ub4 index, d
 	zval_dtor(param->parameter);
 
 	Z_STRLEN_P(param->parameter) = param->max_value_len;
-	Z_STRVAL_P(param->parameter) = emalloc(Z_STRLEN_P(param->parameter)+1);
+	Z_STRVAL_P(param->parameter) = ecalloc(1, Z_STRLEN_P(param->parameter)+1);
 	P->used_for_output = 1;
 
-	P->actual_len = Z_STRLEN_P(param->parameter);	
+	P->actual_len = Z_STRLEN_P(param->parameter);
 	*alenpp = &P->actual_len;
 	*bufpp = Z_STRVAL_P(param->parameter);
 	*piecep = OCI_ONE_PIECE;
@@ -243,7 +264,7 @@ static int oci_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *pa
 	if (param->is_param) {
 		pdo_oci_bound_param *P;
 		sb4 value_sz = -1;
-		
+
 		P = (pdo_oci_bound_param*)param->driver_data;
 
 		switch (event_type) {
@@ -257,7 +278,7 @@ static int oci_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *pa
 			case PDO_PARAM_EVT_ALLOC:
 				P = (pdo_oci_bound_param*)ecalloc(1, sizeof(pdo_oci_bound_param));
 				param->driver_data = P;
-			
+
 				/* figure out what we're doing */
 				switch (PDO_PARAM_TYPE(param->param_type)) {
 					case PDO_PARAM_STMT:
@@ -272,13 +293,13 @@ static int oci_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *pa
 					case PDO_PARAM_STR:
 					default:
 						P->oci_type = SQLT_CHR;
-						value_sz = param->max_value_len + 1;
+						value_sz = param->max_value_len;
 						if (param->max_value_len == 0) {
-							value_sz = 4000; /* maximum size before value is interpreted as a LONG value */
+							value_sz = 1332; /* maximum size before value is interpreted as a LONG value */
 						}
-						
+
 				}
-				
+
 				if (param->name) {
 					STMT_CALL(OCIBindByName, (S->stmt,
 							&P->bind, S->err, (text*)param->name,
@@ -292,7 +313,7 @@ static int oci_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *pa
 							&P->indicator, 0, &P->retcode, 0, 0,
 							OCI_DATA_AT_EXEC));
 				}
-			
+
 				STMT_CALL(OCIBindDynamic, (P->bind,
 							S->err,
 							param, oci_bind_input_cb,
@@ -404,7 +425,7 @@ static int oci_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *pa
 				return 1;
 		}
 	}
-	
+
 	return 1;
 } /* }}} */
 
@@ -442,7 +463,7 @@ static int oci_stmt_fetch(pdo_stmt_t *stmt, enum pdo_fetch_orientation ori,	long
 	if (S->last_err == OCI_SUCCESS_WITH_INFO || S->last_err == OCI_SUCCESS) {
 		return 1;
 	}
-	
+
 	oci_stmt_error("OCIStmtFetch");
 
 	return 0;
@@ -460,6 +481,7 @@ static sb4 oci_define_callback(dvoid *octxp, OCIDefine *define, ub4 iter, dvoid
 			*piecep = OCI_ONE_PIECE;
 			*bufpp = col->data;
 			*alenpp = &col->datalen;
+			*indpp = (dvoid *)&col->indicator;
 			break;
 
 		default:
@@ -510,7 +532,7 @@ static int oci_stmt_describe(pdo_stmt_t *stmt, int colno TSRMLS_DC) /* {{{ */
 	col->name = estrndup(colname, namelen);
 
 	S->cols[colno].dtype = dtype;
-	
+
 	/* how much room do we need to store the field */
 	switch (dtype) {
 		case SQLT_LBI:
@@ -532,7 +554,7 @@ static int oci_stmt_describe(pdo_stmt_t *stmt, int colno TSRMLS_DC) /* {{{ */
 			S->cols[colno].datalen = sizeof(OCILobLocator*);
 			dyn = TRUE;
 			break;
-																																
+
 		case SQLT_BIN:
 		default:
 			if (dtype == SQLT_DAT || dtype == SQLT_NUM
@@ -570,7 +592,7 @@ static int oci_stmt_describe(pdo_stmt_t *stmt, int colno TSRMLS_DC) /* {{{ */
 		STMT_CALL(OCIDefineDynamic, (S->cols[colno].def, S->err, &S->cols[colno],
 				oci_define_callback));
 	}
-	
+
 	return 1;
 } /* }}} */
 
@@ -596,7 +618,7 @@ static size_t oci_blob_write(php_stream *stream, const char *buf, size_t count T
 	if (r != OCI_SUCCESS) {
 		return (size_t)-1;
 	}
-	
+
 	self->offset += amt;
 	return amt;
 }
@@ -642,12 +664,14 @@ static int oci_blob_flush(php_stream *stream TSRMLS_DC)
 	return 0;
 }
 
+/* TODO: implement
 static int oci_blob_seek(php_stream *stream, off_t offset, int whence, off_t *newoffset TSRMLS_DC)
 {
 	struct oci_lob_self *self = (struct oci_lob_self*)stream->abstract;
-	/* TODO: implement */
-	return -1;	
+
+	return -1;
 }
+*/
 
 static php_stream_ops oci_blob_stream_ops = {
 	oci_blob_write,
@@ -703,7 +727,7 @@ static int oci_stmt_get_col(pdo_stmt_t *stmt, int colno, char **ptr, unsigned lo
 			*len = 0;
 			return *ptr ? 1 : 0;
 		}
-		
+
 		*ptr = C->data;
 		*len = C->fetched_len;
 		return 1;
