@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2007 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: formatted_print.c,v 1.82.2.1.2.1 2006/06/26 18:48:56 bjori Exp $ */
+/* $Id: formatted_print.c,v 1.82.2.1.2.16 2007/06/03 09:12:04 shire Exp $ */
 
 #include <math.h>				/* modf() */
 #include "php.h"
@@ -27,6 +27,9 @@
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
+#define LCONV_DECIMAL_POINT (*lconv->decimal_point)
+#else
+#define LCONV_DECIMAL_POINT '.'
 #endif
 
 #define ALIGN_LEFT 0
@@ -50,111 +53,7 @@
 static char hexchars[] = "0123456789abcdef";
 static char HEXCHARS[] = "0123456789ABCDEF";
 
-
-/*
- * cvt.c - IEEE floating point formatting routines for FreeBSD
- * from GNU libc-4.6.27
- */
-
-/*
- *    php_convert_to_decimal converts to decimal
- *      the number of digits is specified by ndigit
- *      decpt is set to the position of the decimal point
- *      sign is set to 0 for positive, 1 for negative
- */
-static char *php_convert_to_decimal(double arg, int ndigits, int *decpt, int *sign, int eflag)
-{
-	register int r2;
-	int mvl;
-	double fi, fj;
-	register char *p, *p1;
-	/*THREADX*/
-#ifndef THREAD_SAFE
-	static char cvt_buf[NDIG];
-#endif
-
-	if (ndigits >= NDIG - 1)
-		ndigits = NDIG - 2;
-	r2 = 0;
-	*sign = 0;
-	p = &cvt_buf[0];
-	if (arg < 0) {
-		*sign = 1;
-		arg = -arg;
-	}
-	arg = modf(arg, &fi);
-	p1 = &cvt_buf[NDIG];
-	/*
-	 * Do integer part
-	 */
-	if (fi != 0) {
-		p1 = &cvt_buf[NDIG];
-		while (fi != 0) {
-			fj = modf(fi / 10, &fi);
-			if (p1 <= &cvt_buf[0]) {
-				mvl = NDIG - ndigits;
-				memmove(&cvt_buf[mvl], &cvt_buf[0], NDIG-mvl-1);
-				p1 += mvl;
-			}
-			*--p1 = (int) ((fj + .03) * 10) + '0';
-			r2++;
-		}
-		while (p1 < &cvt_buf[NDIG])
-			*p++ = *p1++;
-	} else if (arg > 0) {
-		while ((fj = arg * 10) < 1) {
-			if (!eflag && (r2 * -1) < ndigits) {
-				break;
-			}
-			arg = fj;
-			r2--;
-		}
-	}
-	p1 = &cvt_buf[ndigits];
-	if (eflag == 0)
-		p1 += r2;
-	*decpt = r2;
-	if (p1 < &cvt_buf[0]) {
-		cvt_buf[0] = '\0';
-		return (cvt_buf);
-	}
-	if (p <= p1 && p < &cvt_buf[NDIG]) {
-		arg = modf(arg * 10, &fj);
-		if ((int)fj==10) {
-			*p++ = '1';
-			fj = 0;
-			*decpt = ++r2;
-		}
-		while (p <= p1 && p < &cvt_buf[NDIG]) {
-		*p++ = (int) fj + '0';
-			arg = modf(arg * 10, &fj);
-		}
-	}
-	if (p1 >= &cvt_buf[NDIG]) {
-		cvt_buf[NDIG - 1] = '\0';
-		return (cvt_buf);
-	}
-	p = p1;
-	*p1 += 5;
-	while (*p1 > '9') {
-		*p1 = '0';
-		if (p1 > cvt_buf)
-			++ * --p1;
-		else {
-			*p1 = '1';
-			(*decpt)++;
-			if (eflag == 0) {
-				if (p > cvt_buf)
-					*p = '0';
-				p++;
-			}
-		}
-	}
-	*p = '\0';
-	return (cvt_buf);
-}
-
-
+/* php_spintf_appendchar() {{{ */
 inline static void
 php_sprintf_appendchar(char **buffer, int *pos, int *size, char add TSRMLS_DC)
 {
@@ -166,8 +65,9 @@ php_sprintf_appendchar(char **buffer, int *pos, int *size, char add TSRMLS_DC)
 	PRINTF_DEBUG(("sprintf: appending '%c', pos=\n", add, *pos));
 	(*buffer)[(*pos)++] = add;
 }
+/* }}} */
 
-
+/* php_spintf_appendstring() {{{ */
 inline static void
 php_sprintf_appendstring(char **buffer, int *pos, int *size, char *add,
 						   int min_width, int max_width, char padding,
@@ -216,8 +116,9 @@ php_sprintf_appendstring(char **buffer, int *pos, int *size, char *add,
 		}
 	}
 }
+/* }}} */
 
-
+/* php_spintf_appendint() {{{ */
 inline static void
 php_sprintf_appendint(char **buffer, int *pos, int *size, long number,
 						int width, char padding, int alignment, 
@@ -259,7 +160,9 @@ php_sprintf_appendint(char **buffer, int *pos, int *size, long number,
 							 padding, alignment, (NUM_BUF_SIZE - 1) - i,
 							 neg, 0, always_sign);
 }
+/* }}} */
 
+/* php_spintf_appenduint() {{{ */
 inline static void
 php_sprintf_appenduint(char **buffer, int *pos, int *size,
 					   unsigned long number,
@@ -271,7 +174,7 @@ php_sprintf_appenduint(char **buffer, int *pos, int *size,
 
 	PRINTF_DEBUG(("sprintf: appenduint(%x, %x, %x, %d, %d, '%c', %d)\n",
 				  *buffer, pos, size, number, width, padding, alignment));
-	magn = (unsigned int) number;
+	magn = (unsigned long) number;
 
 	/* Can't right-pad 0's on integers */
 	if (alignment == 0 && padding == '0') padding = ' ';
@@ -289,7 +192,9 @@ php_sprintf_appenduint(char **buffer, int *pos, int *size,
 	php_sprintf_appendstring(buffer, pos, size, &numbuf[i], width, 0,
 							 padding, alignment, (NUM_BUF_SIZE - 1) - i, 0, 0, 0);
 }
+/* }}} */
 
+/* php_spintf_appenddouble() {{{ */
 inline static void
 php_sprintf_appenddouble(char **buffer, int *pos,
 						 int *size, double number,
@@ -299,18 +204,11 @@ php_sprintf_appenddouble(char **buffer, int *pos,
 						 int always_sign
 						 TSRMLS_DC)
 {
-	char numbuf[NUM_BUF_SIZE];
-	char *cvt;
-	register int i = 0, j = 0;
-	int sign, decpt, cvt_len;
-	char decimal_point = '.';
+	char num_buf[NUM_BUF_SIZE];
+	char *s = NULL;
+	int s_len = 0, is_negative = 0;
 #ifdef HAVE_LOCALE_H
-	struct lconv lc;
-	char locale_decimal_point;
-	localeconv_r(&lc);
-	locale_decimal_point = (lc.decimal_point)[0];
-#else
-	char locale_decimal_point = '.';
+	struct lconv *lconv;
 #endif
 
 	PRINTF_DEBUG(("sprintf: appenddouble(%x, %x, %x, %f, %d, '%c', %d, %c)\n",
@@ -322,95 +220,71 @@ php_sprintf_appenddouble(char **buffer, int *pos,
 	}
 	
 	if (zend_isnan(number)) {
-		sign = (number<0);
+		is_negative = (number<0);
 		php_sprintf_appendstring(buffer, pos, size, "NaN", 3, 0, padding,
-								 alignment, precision, sign, 0, always_sign);
+								 alignment, precision, is_negative, 0, always_sign);
 		return;
 	}
 
 	if (zend_isinf(number)) {
-		sign = (number<0);
+		is_negative = (number<0);
 		php_sprintf_appendstring(buffer, pos, size, "INF", 3, 0, padding,
-								 alignment, precision, sign, 0, always_sign);
+								 alignment, precision, is_negative, 0, always_sign);
 		return;
 	}
 
-	cvt = php_convert_to_decimal(number, precision, &decpt, &sign, (fmt == 'e'));
-	cvt_len = strlen(cvt);
+	switch (fmt) {			
+		case 'e':
+		case 'E':
+		case 'f':
+		case 'F':
+#ifdef HAVE_LOCALE_H
+			lconv = localeconv();
+#endif
+			s = php_conv_fp((fmt == 'f')?'F':fmt, number, 0, precision,
+						(fmt == 'f')?LCONV_DECIMAL_POINT:'.',
+						&is_negative, &num_buf[1], &s_len);
+			if (is_negative) {
+				num_buf[0] = '-';
+				s = num_buf;
+				s_len++;
+			} else if (always_sign) {
+				num_buf[0] = '+';
+				s = num_buf;
+				s_len++;
+			}
+			break;
 
-	if (sign) {
-		numbuf[i++] = '-';
-	} else if (always_sign) {
-		numbuf[i++] = '+';
+		case 'g':
+		case 'G':
+			if (precision == 0)
+				precision = 1;
+			/*
+			 * * We use &num_buf[ 1 ], so that we have room for the sign
+			 */
+#ifdef HAVE_LOCALE_H
+			lconv = localeconv();
+#endif
+			s = php_gcvt(number, precision, LCONV_DECIMAL_POINT, (fmt == 'G')?'E':'e', &num_buf[1]);
+			is_negative = 0;
+			if (*s == '-') {
+				is_negative = 1;
+				s = &num_buf[1];
+			} else if (always_sign) {
+				num_buf[0] = '+';
+				s = num_buf;
+			}
+
+			s_len = strlen(s);
+			break;
 	}
 
-	if (fmt == 'f' || fmt == 'F') {
-		if (decpt <= 0) {
-			numbuf[i++] = '0';
-			if (precision > 0) {
-				int k = precision;
-				numbuf[i++] = fmt == 'F' ? decimal_point : locale_decimal_point;
-				while ((decpt++ < 0) && k--) {
-					numbuf[i++] = '0';
-				}
-			}
-		} else {
-			while (decpt-- > 0) {
-				numbuf[i++] = j < cvt_len ? cvt[j++] : '0';
-			}
-			if (precision > 0) {
-				numbuf[i++] = fmt == 'F' ? decimal_point : locale_decimal_point;
-				while (precision-- > 0) {
-					numbuf[i++] = j < cvt_len ? cvt[j++] : '0';
-				}
-			}
-		}
-	} else if (fmt == 'e' || fmt == 'E') {
-		char *exp_p;
-		int dec2;
-		
-		decpt--;
-		
-		numbuf[i++] = cvt[j++];
-		numbuf[i++] = decimal_point;	
-
-		if (precision > 0) {
-			int k = precision;
-				
-			while (k-- && cvt[j]) {
-				numbuf[i++] = cvt[j++];
-			}
-		} else {
-			numbuf[i++] = '0';
-		}
-		
-		numbuf[i++] = fmt;
-		exp_p = php_convert_to_decimal(decpt, 0, &dec2, &sign, 0);
-		numbuf[i++] = sign ? '-' : '+';
-		if (*exp_p) { 
-			while (*exp_p) {
-				numbuf[i++] = *(exp_p++);
-			}
-		} else {
-			numbuf[i++] = '0';
-		}
-	} else {
-		numbuf[i++] = cvt[j++];
-		if (precision > 0)
-			numbuf[i++] = decimal_point;
-	}
-
-	while (cvt[j]) {
-		numbuf[i++] = cvt[j++];
-	}
-
-	numbuf[i] = '\0';
-
-	php_sprintf_appendstring(buffer, pos, size, numbuf, width, 0, padding,
-							 alignment, i, sign, 0, always_sign);
+	php_sprintf_appendstring(buffer, pos, size, s, width, 0, padding,
+							 alignment, s_len, is_negative, 0, always_sign);
 }
+/* }}} */
 
-
+/* php_spintf_appendd2n() {{{ */
 inline static void
 php_sprintf_append2n(char **buffer, int *pos, int *size, long number,
 					 int width, char padding, int alignment, int n,
@@ -439,9 +313,10 @@ php_sprintf_append2n(char **buffer, int *pos, int *size, long number,
 							 padding, alignment, (NUM_BUF_SIZE - 1) - i,
 							 0, expprec, 0);
 }
+/* }}} */
 
-
-inline static long
+/* php_spintf_getnumber() {{{ */
+inline static int
 php_sprintf_getnumber(char *buffer, int *pos)
 {
 	char *endptr;
@@ -453,10 +328,16 @@ php_sprintf_getnumber(char *buffer, int *pos)
 	}
 	PRINTF_DEBUG(("sprintf_getnumber: number was %d bytes long\n", i));
 	*pos += i;
-	return num;
-}
 
-/* {{{ php_formatted_print
+	if (num >= INT_MAX || num < 0) {
+		return -1;
+	} else {
+		return (int) num;
+	}
+}
+/* }}} */
+
+/* php_formatted_print() {{{
  * New sprintf implementation for PHP.
  *
  * Modifiers:
@@ -485,7 +366,7 @@ php_formatted_print(int ht, int *len, int use_array, int format_offset TSRMLS_DC
 {
 	zval ***args, **z_format;
 	int argc, size = 240, inpos = 0, outpos = 0, temppos;
-	int alignment, width, precision, currarg, adjusting, argnum;
+	int alignment, currarg, adjusting, argnum, width, precision;
 	char *format, *result, padding;
 	int always_sign;
 
@@ -561,10 +442,10 @@ php_formatted_print(int ht, int *len, int use_array, int format_offset TSRMLS_DC
 				if (format[temppos] == '$') {
 					argnum = php_sprintf_getnumber(format, &inpos);
 
-					if (argnum == 0) {
+					if (argnum <= 0) {
 						efree(result);
 						efree(args);
-						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Zero is not a valid argument number");
+						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Argument number must be greater than zero.");
 						return NULL;
 					}
 
@@ -603,7 +484,12 @@ php_formatted_print(int ht, int *len, int use_array, int format_offset TSRMLS_DC
 				/* after modifiers comes width */
 				if (isdigit((int)format[inpos])) {
 					PRINTF_DEBUG(("sprintf: getting width\n"));
-					width = php_sprintf_getnumber(format, &inpos);
+					if ((width = php_sprintf_getnumber(format, &inpos)) < 0) {
+						efree(result);
+						efree(args);
+						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Width must be greater than zero and less than %d.", INT_MAX);
+						return NULL;
+					}
 					adjusting |= ADJ_WIDTH;
 				} else {
 					width = 0;
@@ -615,7 +501,12 @@ php_formatted_print(int ht, int *len, int use_array, int format_offset TSRMLS_DC
 					inpos++;
 					PRINTF_DEBUG(("sprintf: getting precision\n"));
 					if (isdigit((int)format[inpos])) {
-						precision = php_sprintf_getnumber(format, &inpos);
+						if ((precision = php_sprintf_getnumber(format, &inpos)) < 0) {
+							efree(result);
+							efree(args);
+							php_error_docref(NULL TSRMLS_CC, E_WARNING, "Precision must be greater than zero and less than %d.", INT_MAX);
+							return NULL;
+						}
 						adjusting |= ADJ_PRECISION;
 						expprec = 1;
 					} else {
@@ -690,10 +581,12 @@ php_formatted_print(int ht, int *len, int use_array, int format_offset TSRMLS_DC
 										  width, padding, alignment);
 					break;
 
+				case 'g':
+				case 'G':
 				case 'e':
+				case 'E':
 				case 'f':
 				case 'F':
-					/* XXX not done */
 					convert_to_double(tmp);
 					php_sprintf_appenddouble(&result, &outpos, &size,
 											 Z_DVAL_P(tmp),
@@ -886,7 +779,6 @@ PHP_FUNCTION(vfprintf)
 }
 /* }}} */
 
-
 /*
  * Local variables:
  * tab-width: 4
