@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: info.c,v 1.249.2.10.2.9 2007/01/26 15:33:18 tony2001 Exp $ */
+/* $Id: info.c,v 1.249.2.10.2.14 2007/07/21 01:24:26 jani Exp $ */
 
 #include "php.h"
 #include "php_ini.h"
@@ -76,13 +76,9 @@ static int php_info_write_wrapper(const char *str, uint str_length)
 }
 
 
-/* {{{ _display_module_info
- */
-static int _display_module_info(zend_module_entry *module, void *arg TSRMLS_DC)
+PHPAPI void php_info_print_module(zend_module_entry *module TSRMLS_DC) /* {{{ */
 {
-	int show_info_func = *((int *) arg);
-
-	if (show_info_func && module->info_func) {
+	if (module->info_func) {
 		if (!sapi_module.phpinfo_as_text) {
 			php_printf("<h2><a name=\"module_%s\">%s</a></h2>\n", module->name, module->name);
 		} else {
@@ -91,7 +87,7 @@ static int _display_module_info(zend_module_entry *module, void *arg TSRMLS_DC)
 			php_info_print_table_end();
 		}
 		module->info_func(module TSRMLS_CC);
-	} else if (!show_info_func && !module->info_func) {
+	} else {
 		if (!sapi_module.phpinfo_as_text) {
 			php_printf("<tr>");
 			php_printf("<td>");
@@ -102,7 +98,24 @@ static int _display_module_info(zend_module_entry *module, void *arg TSRMLS_DC)
 			php_printf("\n");
 		}	
 	}
-	return 0;
+}
+/* }}} */
+
+static int _display_module_info_func(zend_module_entry *module TSRMLS_DC) /* {{{ */
+{
+	if (module->info_func) {
+		php_info_print_module(module TSRMLS_CC);
+	}
+	return ZEND_HASH_APPLY_KEEP;
+}
+/* }}} */
+
+static int _display_module_info_def(zend_module_entry *module TSRMLS_DC) /* {{{ */
+{
+	if (!module->info_func) {
+		php_info_print_module(module TSRMLS_CC);
+	}
+	return ZEND_HASH_APPLY_KEEP;
 }
 /* }}} */
 
@@ -461,7 +474,8 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 		php_info_print_table_row(2, "Virtual Directory Support", "disabled" );
 #endif
 
-		php_info_print_table_row(2, "Configuration File (php.ini) Path", php_ini_opened_path?php_ini_opened_path:PHP_CONFIG_FILE_PATH);
+		php_info_print_table_row(2, "Configuration File (php.ini) Path", PHP_CONFIG_FILE_PATH);
+		php_info_print_table_row(2, "Loaded Configuration File", php_ini_opened_path ? php_ini_opened_path : "(none)");
 
 		if (strlen(PHP_CONFIG_FILE_SCAN_DIR)) {
 			php_info_print_table_row(2, "Scan this dir for additional .ini files", PHP_CONFIG_FILE_SCAN_DIR);
@@ -662,22 +676,19 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 	}
 
 	if (flag & PHP_INFO_MODULES) {
-		int show_info_func;
 		HashTable sorted_registry;
 		zend_module_entry tmp;
 
-		zend_hash_init(&sorted_registry, 50, NULL, NULL, 1);
+		zend_hash_init(&sorted_registry, zend_hash_num_elements(&module_registry), NULL, NULL, 1);
 		zend_hash_copy(&sorted_registry, &module_registry, NULL, &tmp, sizeof(zend_module_entry));
 		zend_hash_sort(&sorted_registry, zend_qsort, module_name_cmp, 0 TSRMLS_CC);
 
-		show_info_func = 1;
-		zend_hash_apply_with_argument(&sorted_registry, (apply_func_arg_t) _display_module_info, &show_info_func TSRMLS_CC);
+		zend_hash_apply(&sorted_registry, (apply_func_t) _display_module_info_func TSRMLS_CC);
 
 		SECTION("Additional Modules");
 		php_info_print_table_start();
 		php_info_print_table_header(1, "Module Name");
-		show_info_func = 0;
-		zend_hash_apply_with_argument(&sorted_registry, (apply_func_arg_t) _display_module_info, &show_info_func TSRMLS_CC);
+		zend_hash_apply(&sorted_registry, (apply_func_t) _display_module_info_def TSRMLS_CC);
 		php_info_print_table_end();
 
 		zend_hash_destroy(&sorted_registry);
@@ -770,7 +781,7 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 /* }}} */
 
 
-PHPAPI void php_info_print_table_start()
+PHPAPI void php_info_print_table_start(void)
 {
 	if (!sapi_module.phpinfo_as_text) {
 		php_printf("<table border=\"0\" cellpadding=\"3\" width=\"600\">\n");
@@ -779,7 +790,7 @@ PHPAPI void php_info_print_table_start()
 	}	
 }
 
-PHPAPI void php_info_print_table_end()
+PHPAPI void php_info_print_table_end(void)
 {
 	if (!sapi_module.phpinfo_as_text) {
 		php_printf("</table><br />\n");
@@ -803,7 +814,7 @@ PHPAPI void php_info_print_box_start(int flag)
 	}
 }
 
-PHPAPI void php_info_print_box_end()
+PHPAPI void php_info_print_box_end(void)
 {
 	if (!sapi_module.phpinfo_as_text) {
 		php_printf("</td></tr>\n");
@@ -811,7 +822,7 @@ PHPAPI void php_info_print_box_end()
 	php_info_print_table_end();
 }
 
-PHPAPI void php_info_print_hr()
+PHPAPI void php_info_print_hr(void)
 {
 	if (!sapi_module.phpinfo_as_text) {
 		php_printf("<hr />\n");
@@ -1045,7 +1056,7 @@ PHP_FUNCTION(phpcredits)
 
 /* {{{ php_logo_guid
  */
-PHPAPI char *php_logo_guid()
+PHPAPI char *php_logo_guid(void)
 {
 	char *logo_guid;
 
@@ -1055,7 +1066,7 @@ PHPAPI char *php_logo_guid()
 	the_time = time(NULL);
 	ta = php_localtime_r(&the_time, &tmbuf);
 
-	if ((ta->tm_mon==3) && (ta->tm_mday==1)) {
+	if (ta && (ta->tm_mon==3) && (ta->tm_mday==1)) {
 		logo_guid = PHP_EGG_LOGO_GUID;
 	} else {
 		logo_guid = PHP_LOGO_GUID;
@@ -1159,6 +1170,18 @@ PHP_FUNCTION(php_ini_scanned_files)
 }
 /* }}} */
 
+/* {{{ proto string php_ini_loaded_file(void)
+   Return the actual loaded ini filename */
+PHP_FUNCTION(php_ini_loaded_file)
+{
+	if (php_ini_opened_path) {
+		RETURN_STRING(php_ini_opened_path, 1);
+	} else {
+		RETURN_FALSE;
+	}
+}
+/* }}} */
+
 /*
  * Local variables:
  * tab-width: 4
