@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2007 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: xsltprocessor.c,v 1.39.2.2.2.4 2006/07/31 13:05:35 chregu Exp $ */
+/* $Id: xsltprocessor.c,v 1.39.2.2.2.9 2007/07/30 16:33:22 rrichards Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -87,7 +87,7 @@ static char **php_xsl_xslt_make_params(HashTable *parht, int xpath_params TSRMLS
 	int i = 0;
 
 	parsize = (2 * zend_hash_num_elements(parht) + 1) * sizeof(char *);
-	params = (char **)emalloc(parsize);
+	params = (char **)safe_emalloc((2 * zend_hash_num_elements(parht) + 1), sizeof(char *), 0);
 	memset((char *)params, 0, parsize);
 
 	for (zend_hash_internal_pointer_reset(parht);
@@ -268,7 +268,7 @@ static void xsl_ext_function_php(xmlXPathParserContextPtr ctxt, int nargs, int t
 		
 	} else if ( intern->registerPhpFunctions == 2 && zend_hash_exists(intern->registered_phpfunctions, callable, strlen(callable) + 1) == 0) { 
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Not allowed to call handler '%s()'.", callable);
-		// Push an empty string, so that we at least have an xslt result...
+		/* Push an empty string, so that we at least have an xslt result... */
 		valuePush(ctxt, xmlXPathNewString(""));
 	} else {
 		result = zend_call_function(&fci, NULL TSRMLS_CC);
@@ -384,13 +384,15 @@ PHP_FUNCTION(xsl_xsltprocessor_import_stylesheet)
 	efree(member);
 	if (clone_docu == 0) {
 		/* check if the stylesheet is using xsl:key, if yes, we have to clone the document _always_ before a transformation */
-		nodep = xmlDocGetRootElement(sheetp->doc)->children;
-		while (nodep) {
-			if (nodep->type == XML_ELEMENT_NODE && xmlStrEqual(nodep->name, "key") && xmlStrEqual(nodep->ns->href, XSLT_NAMESPACE)) {
-				intern->hasKeys = 1;
-				break;
+		nodep = xmlDocGetRootElement(sheetp->doc);
+		if (nodep && (nodep = nodep->children)) {
+			while (nodep) {
+				if (nodep->type == XML_ELEMENT_NODE && xmlStrEqual(nodep->name, "key") && xmlStrEqual(nodep->ns->href, XSLT_NAMESPACE)) {
+					intern->hasKeys = 1;
+					break;
+				}
+				nodep = nodep->next;
 			}
-			nodep = nodep->next;
 		}
 	} else {
 		intern->hasKeys = clone_docu;
@@ -503,21 +505,49 @@ PHP_FUNCTION(xsl_xsltprocessor_transform_to_doc)
 	zval *id, *rv = NULL, *docp = NULL;
 	xmlDoc *newdocp;
 	xsltStylesheetPtr sheetp;
-	int ret;
+	int ret, ret_class_len=0;
+	char *ret_class = NULL;
 	xsl_object *intern;
-	
+
 	id = getThis();
 	intern = (xsl_object *)zend_object_store_get_object(id TSRMLS_CC);
 	sheetp = (xsltStylesheetPtr) intern->ptr;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "o", &docp) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "o|s!", &docp, &ret_class, &ret_class_len) == FAILURE) {
 		RETURN_FALSE;
 	}
 
 	newdocp = php_xsl_apply_stylesheet(id, intern, sheetp, docp TSRMLS_CC);
 
 	if (newdocp) {
-		DOM_RET_OBJ(rv, (xmlNodePtr) newdocp, &ret, NULL);
+		if (ret_class) {
+			int found;
+			char *curclass_name;
+			zend_class_entry *curce, **ce;
+			php_libxml_node_object *interndoc;
+
+			curce = Z_OBJCE_P(docp);
+			curclass_name = curce->name;
+			while (curce->parent != NULL) {
+				curce = curce->parent;
+			}
+
+			found = zend_lookup_class(ret_class, ret_class_len, &ce TSRMLS_CC);
+			if ((found != SUCCESS) || !instanceof_function(*ce, curce TSRMLS_CC)) {
+				xmlFreeDoc(newdocp);
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, 
+					"Expecting class compatible with %s, '%s' given", curclass_name, ret_class);
+				RETURN_FALSE;
+			}
+
+			object_init_ex(return_value, *ce);
+		
+			interndoc = (php_libxml_node_object *)zend_objects_get_address(return_value TSRMLS_CC);
+			php_libxml_increment_doc_ref(interndoc, newdocp TSRMLS_CC);
+			php_libxml_increment_node_ptr(interndoc, (xmlNodePtr)newdocp, (void *)interndoc TSRMLS_CC);
+		} else {
+			DOM_RET_OBJ(rv, (xmlNodePtr) newdocp, &ret, NULL);
+		}
 	} else {
 		RETURN_FALSE;
 	}
