@@ -16,15 +16,17 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: php_open_temporary_file.h,v 1.13.2.1.2.1 2007/01/01 09:36:11 sebastian Exp $ */
+/* $Id: php_open_temporary_file.h,v 1.13.2.1.2.4 2007/07/21 01:43:33 jani Exp $ */
 
 #ifndef PHP_OPEN_TEMPORARY_FILE_H
 #define PHP_OPEN_TEMPORARY_FILE_H
 
 BEGIN_EXTERN_C()
 PHPAPI FILE *php_open_temporary_file(const char *dir, const char *pfx, char **opened_path_p TSRMLS_DC);
+PHPAPI int php_open_temporary_fd_ex(const char *dir, const char *pfx, char **opened_path_p, zend_bool open_basedir_check TSRMLS_DC);
 PHPAPI int php_open_temporary_fd(const char *dir, const char *pfx, char **opened_path_p TSRMLS_DC);
 PHPAPI const char *php_get_temporary_directory(void);
+PHPAPI void php_shutdown_temporary_directory(void);
 END_EXTERN_C()
 
 #endif /* PHP_OPEN_TEMPORARY_FILE_H */
