@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2007 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: spprintf.c,v 1.25.2.2.2.1 2006/05/07 12:40:17 helly Exp $ */
+/* $Id: spprintf.c,v 1.25.2.2.2.10 2007/08/03 14:31:28 tony2001 Exp $ */
 
 /* This is the spprintf implementation.
  * It has emerged from apache snprintf. See original header:
@@ -76,7 +76,6 @@
  * SIO stdio-replacement strx_* functions by Panos Tsirigotis
  * <panos@alumni.cs.colorado.edu> for xinetd.
  */
-
 #include "php.h"
 
 #include <stddef.h>
@@ -91,6 +90,13 @@
 #include <inttypes.h>
 #endif
 
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#define LCONV_DECIMAL_POINT (*lconv->decimal_point)
+#else
+#define LCONV_DECIMAL_POINT '.'
+#endif
+
 #include "snprintf.h"
 
 #define FALSE           0
@@ -106,6 +112,8 @@
 
 #include "ext/standard/php_smart_str.h"
 
+/* {{{ macros */
+
 /*
  * NUM_BUF_SIZE is the size of the buffer used for arithmetic conversions
  *
@@ -170,12 +178,12 @@
 		}												\
 } while (0)
 
-
+/* }}} */
 
 /*
  * Do format conversion placing the output in buffer
  */
-static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap)
+static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap) /* {{{ */
 {
 	register char *s = NULL;
 	char *q;
@@ -196,6 +204,10 @@ static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap)
 	char num_buf[NUM_BUF_SIZE];
 	char char_buf[2];			/* for printing %% and %<unknown> */
 
+#ifdef HAVE_LOCALE_H
+	struct lconv *lconv = NULL;
+#endif
+
 	/*
 	 * Flag variables
 	 */
@@ -292,6 +304,25 @@ static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap)
 					fmt++;
 					modifier = LM_LONG_DOUBLE;
 					break;
+				case 'I':
+					fmt++;
+#if SIZEOF_LONG_LONG
+					if (*fmt == '6' && *(fmt+1) == '4') {
+						fmt += 2;
+						modifier = LM_LONG_LONG;
+					} else
+#endif
+						if (*fmt == '3' && *(fmt+1) == '2') {
+							fmt += 2;
+							modifier = LM_LONG;
+						} else {
+#ifdef _WIN64
+							modifier = LM_LONG_LONG;
+#else
+							modifier = LM_LONG;
+#endif
+						}
+					break;
 				case 'l':
 					fmt++;
 #if SIZEOF_LONG_LONG
@@ -528,6 +559,7 @@ static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap)
 
 
 				case 'f':
+				case 'F':
 				case 'e':
 				case 'E':
 					switch(modifier) {
@@ -548,8 +580,14 @@ static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap)
 						s = "inf";
 						s_len = 3;
 					} else {
-						s = ap_php_conv_fp(*fmt, fp_num, alternate_form,
-									(adjust_precision == NO) ? FLOAT_DIGITS : precision,
+#ifdef HAVE_LOCALE_H
+						if (!lconv) {
+							lconv = localeconv();
+						}
+#endif
+						s = php_conv_fp((*fmt == 'f')?'F':*fmt, fp_num, alternate_form,
+						 (adjust_precision == NO) ? FLOAT_DIGITS : precision,
+						 (*fmt == 'f')?LCONV_DECIMAL_POINT:'.',
 									&is_negative, &num_buf[1], &s_len);
 						if (is_negative)
 							prefix_char = '-';
@@ -563,6 +601,7 @@ static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap)
 
 				case 'g':
 				case 'G':
+				case 'H':
 					switch(modifier) {
 						case LM_LONG_DOUBLE:
 							fp_num = (double) va_arg(ap, long double);
@@ -596,8 +635,12 @@ static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap)
 					/*
 					 * * We use &num_buf[ 1 ], so that we have room for the sign
 					 */
-					s = ap_php_gcvt(fp_num, precision, &num_buf[1],
-							alternate_form);
+#ifdef HAVE_LOCALE_H
+					if (!lconv) {
+						lconv = localeconv();
+					}
+#endif
+					s = php_gcvt(fp_num, precision, *fmt=='H' ? '.' : LCONV_DECIMAL_POINT, (*fmt == 'G' || *fmt == 'H')?'E':'e', &num_buf[1]);
 					if (*s == '-')
 						prefix_char = *s++;
 					else if (print_sign)
@@ -609,8 +652,6 @@ static void xbuf_format_converter(smart_str *xbuf, const char *fmt, va_list ap)
 
 					if (alternate_form && (q = strchr(s, '.')) == NULL)
 						s[s_len++] = '.';
-					if (*fmt == 'G' && (q = strchr(s, 'e')) != NULL)
-						*q = 'E';
 					break;
 
 
@@ -715,12 +756,12 @@ skip_output:
 	}
 	return;
 }
-
+/* }}} */
 
 /*
  * This is the general purpose conversion function.
  */
-PHPAPI int vspprintf(char **pbuf, size_t max_len, const char *format, va_list ap)
+PHPAPI int vspprintf(char **pbuf, size_t max_len, const char *format, va_list ap) /* {{{ */
 {
 	smart_str xbuf = {0};
 
@@ -735,9 +776,9 @@ PHPAPI int vspprintf(char **pbuf, size_t max_len, const char *format, va_list ap
 	
 	return xbuf.len;
 }
+/* }}} */
 
-
-PHPAPI int spprintf(char **pbuf, size_t max_len, const char *format, ...)
+PHPAPI int spprintf(char **pbuf, size_t max_len, const char *format, ...) /* {{{ */
 {
 	int cc;
 	va_list ap;
@@ -747,6 +788,7 @@ PHPAPI int spprintf(char **pbuf, size_t max_len, const char *format, ...)
 	va_end(ap);
 	return (cc);
 }
+/* }}} */
 
 /*
  * Local variables:
