@@ -23,7 +23,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: run-tests.php,v 1.226.2.37.2.21 2007/01/01 19:32:09 iliaa Exp $ */
+/* $Id: run-tests.php,v 1.226.2.37.2.34 2007/07/31 21:29:21 jani Exp $ */
 
 /* Sanity check to ensure that pcre extension needed by this script is available.
  * In the event it is not, print a nice error message indicating that this script will
@@ -115,7 +115,7 @@ if (getenv('TEST_PHP_EXECUTABLE')) {
 if (getenv('TEST_PHP_CGI_EXECUTABLE')) {
 	$php_cgi = getenv('TEST_PHP_CGI_EXECUTABLE');
 	if ($php_cgi=='auto') {
-		$php_cgi = $cwd.'/sapi/cgi/php';
+		$php_cgi = $cwd.'/sapi/cgi/php-cgi';
 		putenv("TEST_PHP_CGI_EXECUTABLE=$php_cgi");
 	}
 	$environment['TEST_PHP_CGI_EXECUTABLE'] = $php_cgi;
@@ -160,6 +160,7 @@ $ini_overwrites = array(
 		'output_buffering=Off',
 		'error_reporting=8191',
 		'display_errors=1',
+		'display_startup_errors=1',
 		'log_errors=0',
 		'html_errors=0',
 		'track_errors=1',
@@ -398,14 +399,16 @@ if (isset($argc) && $argc > 1) {
 					$html_output = is_resource($html_file);
 					break;
 				case '--version':
-					echo '$Revision: 1.226.2.37.2.21 $'."\n";
+					echo '$Revision: 1.226.2.37.2.34 $'."\n";
 					exit(1);
+
+				case 'u':
+				case 'U':
+					// Allow using u or U for forward compatibility
+					break;
+					
 				default:
 					echo "Illegal switch '$switch' specified!\n";
-					if ($switch == 'u' || $switch == 'U') {
-						break;
-					}
-					// break
 				case 'h':
 				case '-help':
 				case '--help':
@@ -471,7 +474,18 @@ HELP;
 		}
 		if (!$is_switch) {
 			$testfile = realpath($argv[$i]);
-			if (is_dir($testfile)) {
+			if (!$testfile && strpos($argv[$i], '*') !== false && function_exists('glob')) {
+				if (preg_match("/\.phpt$/", $argv[$i])) {
+					$pattern_match = glob($argv[$i]);
+				} else if (preg_match("/\*$/", $argv[$i])) {
+					$pattern_match = glob($argv[$i] . '.phpt');
+				} else {
+					die("bogus test name " . $argv[$i] . "\n");
+				}
+				if (is_array($pattern_match)) {
+					$test_files = array_merge($test_files, $pattern_match);
+				}
+			} else if (is_dir($testfile)) {
 				find_files($testfile);
 			} else if (preg_match("/\.phpt$/", $testfile)) {
 				$test_files[] = $testfile;
@@ -518,7 +532,7 @@ HELP;
 		if ($html_output) {
 			fclose($html_file);
 		}
-		if (getenv('REPORT_EXIT_STATUS') == 1 and ereg('FAILED( |$)', implode(' ', $test_results))) {
+		if (getenv('REPORT_EXIT_STATUS') == 1 and preg_match('/FAILED(?: |$)/', implode(' ', $test_results))) {
 			exit(1);
 		}
 		exit(0);
@@ -534,7 +548,7 @@ $exts_skipped = 0;
 $ignored_by_ext = 0;
 sort($exts_to_test);
 $test_dirs = array();
-$optionals = array('tests', 'ext', 'Zend', 'ZendEngine2', 'sapi/cli');
+$optionals = array('tests', 'ext', 'Zend', 'ZendEngine2', 'sapi/cli', 'sapi/cgi');
 foreach($optionals as $dir) {
 	if (@filetype($dir) == 'dir') {
 		$test_dirs[] = $dir;
@@ -648,7 +662,10 @@ define('QA_SUBMISSION_PAGE', 'http://qa.php.net/buildtest-process.php');
 /* We got failed Tests, offer the user to send an e-mail to QA team, unless NO_INTERACTION is set */
 if (!getenv('NO_INTERACTION')) {
 	$fp = fopen("php://stdin", "r+");
-	echo "\nYou may have found a problem in PHP.\nWe would like to send this report automatically to the\n";
+	if ($sum_results['FAILED'] || $sum_results['BORKED'] || $sum_results['WARNED'] || $sum_results['LEAKED']) {
+		echo "\nYou may have found a problem in PHP.";
+	}
+	echo "\nWe would like to send this report automatically to the\n";
 	echo "PHP QA team, to give us a better understanding of how\nthe test cases are doing. If you don't want to send it\n";
 	echo "immediately, you can choose \"s\" to save the report to\na file that you can send us later.\n";
 	echo "Do you want to send this report now? [Yns]: ";
@@ -700,8 +717,8 @@ if ($just_save_results || !getenv('NO_INTERACTION')) {
 
 		if (substr(PHP_OS, 0, 3) != "WIN") {
 			/* If PHP_AUTOCONF is set, use it; otherwise, use 'autoconf'. */
-			if (!empty($_ENV['PHP_AUTOCONF'])) {
-				$autoconf = shell_exec($_ENV['PHP_AUTOCONF'] . ' --version');
+			if (getenv('PHP_AUTOCONF')) {
+				$autoconf = shell_exec(getenv('PHP_AUTOCONF') . ' --version');
 			} else {
 				$autoconf = shell_exec('autoconf --version');
 			}
@@ -968,7 +985,7 @@ TEST $file
 		$bork_info = "empty test [$file]";
 		$borked = true;
 	}
-	if (!ereg('^--TEST--',$line,$r)) {
+	if (strncmp('--TEST--', $line, 8)) {
 		$bork_info = "tests must start with --TEST-- [$file]";
 		$borked = true;
 	}
@@ -993,7 +1010,7 @@ TEST $file
 		}
 
 		// End of actual test?
-		if ($secfile && preg_match('/^===DONE===/', $line, $r)) {
+		if ($secfile && preg_match('/^===DONE===$/', $line)) {
 			$secdone = true;
 		}
 	}
@@ -1048,12 +1065,17 @@ TEST $file
 		} elseif (!strncasecmp(PHP_OS, "win", 3) && file_exists(dirname($php) ."/php-cgi.exe")) {
 			$old_php = $php;
 			$php = realpath(dirname($php) ."/php-cgi.exe") .' -C ';
-		} elseif (file_exists("./sapi/cgi/php")) {
-			$old_php = $php;
-			$php = realpath("./sapi/cgi/php") . ' -C ';
 		} else {
-			show_result("SKIP", $tested, $tested_file, "reason: CGI not available");
-			return 'SKIPPED';
+			if (file_exists(dirname($php)."/../../sapi/cgi/php-cgi")) {
+				$old_php = $php;
+				$php = realpath(dirname($php)."/../../sapi/cgi/php-cgi") . ' -C ';
+			} else if (file_exists("./sapi/cgi/php-cgi")) {
+				$old_php = $php;
+				$php = realpath("./sapi/cgi/php-cgi") . ' -C ';
+			} else {
+				show_result("SKIP", $tested, $tested_file, "reason: CGI not available");
+				return 'SKIPPED';
+			}
 		}
 	}
 
@@ -1173,14 +1195,19 @@ TEST $file
 			save_text($test_skipif, $section_text['SKIPIF'], $temp_skipif);
 			$extra = substr(PHP_OS, 0, 3) !== "WIN" ?
 				"unset REQUEST_METHOD; unset QUERY_STRING; unset PATH_TRANSLATED; unset SCRIPT_FILENAME; unset REQUEST_METHOD;": "";
-			$output = system_with_timeout("$extra $php -q $ini_settings $test_skipif", $env);
+
+			if ($leak_check) {
+				$env['USE_ZEND_ALLOC'] = '0';
+			} else {
+				$env['USE_ZEND_ALLOC'] = '1';
+			}
+			$output = system_with_timeout("$extra $php $pass_options -q $ini_settings $test_skipif", $env);
 			if (!$cfg['keep']['skip']) {
 				@unlink($test_skipif);
 			}
-			if (!strncasecmp('skip', trim($output), 4)) {
-				$reason = (eregi("^skip[[:space:]]*(.+)\$", trim($output))) ? eregi_replace("^skip[[:space:]]*(.+)\$", "\\1", trim($output)) : FALSE;
-				if ($reason) {
-					show_result("SKIP", $tested, $tested_file, "reason: $reason", $temp_filenames);
+			if (!strncasecmp('skip', ltrim($output), 4)) {
+				if (preg_match('/^\s*skip\s*(.+)\s*/i', $output, $m)) {
+					show_result("SKIP", $tested, $tested_file, "reason: $m[1]", $temp_filenames);
 				} else {
 					show_result("SKIP", $tested, $tested_file, '', $temp_filenames);
 				}
@@ -1192,17 +1219,15 @@ TEST $file
 				}
 				return 'SKIPPED';
 			}
-			if (!strncasecmp('info', trim($output), 4)) {
-				$reason = (ereg("^info[[:space:]]*(.+)\$", trim($output))) ? ereg_replace("^info[[:space:]]*(.+)\$", "\\1", trim($output)) : FALSE;
-				if ($reason) {
-					$info = " (info: $reason)";
+			if (!strncasecmp('info', ltrim($output), 4)) {
+				if (preg_match('/^\s*info\s*(.+)\s*/i', $output, $m)) {
+					$info = " (info: $m[1])";
 				}
 			}
-			if (!strncasecmp('warn', trim($output), 4)) {
-				$reason = (ereg("^warn[[:space:]]*(.+)\$", trim($output))) ? ereg_replace("^warn[[:space:]]*(.+)\$", "\\1", trim($output)) : FALSE;
-				if ($reason) {
+			if (!strncasecmp('warn', ltrim($output), 4)) {
+				if (preg_match('/^\s*warn\s*(.+)\s*/i', $output, $m)) {
 					$warn = true; /* only if there is a reason */
-					$info = " (warn: $reason)";
+					$info = " (warn: $m[1])";
 				}
 			}
 		}
@@ -1304,8 +1329,8 @@ TEST $file
 
 		$request = '';
 		foreach ($raw_lines as $line) {
-			if (empty($env['CONTENT_TYPE']) && eregi('^(Content-Type:)(.*)', $line, $res)) {
-				$env['CONTENT_TYPE'] = trim(str_replace("\r", '', $res[2]));
+			if (empty($env['CONTENT_TYPE']) && preg_match('/^Content-Type:(.*)/i', $line, $res)) {
+				$env['CONTENT_TYPE'] = trim(str_replace("\r", '', $res[1]));
 				continue;
 			}
 			$request .= $line . "\n";
@@ -1322,6 +1347,15 @@ TEST $file
 	} elseif (array_key_exists('POST', $section_text) && !empty($section_text['POST'])) {
 
 		$post = trim($section_text['POST']);
+
+		if (array_key_exists('GZIP_POST', $section_text) && function_exists('gzencode')) {
+			$post = gzencode($post, 9, FORCE_GZIP);
+			$env['HTTP_CONTENT_ENCODING'] = 'gzip';
+		} else if (array_key_exists('DEFLATE_POST', $section_text) && function_exists('gzcompress')) {
+			$post = gzcompress($post, 9);
+			$env['HTTP_CONTENT_ENCODING'] = 'deflate';
+		}
+
 		save_text($tmp_post, $post);
 		$content_length = strlen($post);
 
@@ -1375,7 +1409,7 @@ COMMAND $cmd
 				settings2params($clean_params);
 				$extra = substr(PHP_OS, 0, 3) !== "WIN" ?
 					"unset REQUEST_METHOD; unset QUERY_STRING; unset PATH_TRANSLATED; unset SCRIPT_FILENAME; unset REQUEST_METHOD;": "";
-				system_with_timeout("$extra $php -q $clean_params $test_clean", $env);
+				system_with_timeout("$extra $php $pass_options -q $clean_params $test_clean", $env);
 			}
 			if (!$cfg['keep']['clean']) {
 				@unlink($test_clean);
@@ -1511,11 +1545,13 @@ COMMAND $cmd
 	}
 
 	if ($leaked) {
-		$restype = 'LEAK';
-	} else if ($warn) {
-		$restype = 'WARN';
-	} else {
-		$restype = 'FAIL';
+		$restype[] = 'LEAK';
+	}
+	if ($warn) {
+		$restype[] = 'WARN';
+	}
+	if (!$passed) {
+		$restype[] = 'FAIL';
 	}
 
 	if (!$passed) {
@@ -1547,21 +1583,23 @@ $output
 		}
 	}
 
-	show_result($restype, $tested, $tested_file, $info, $temp_filenames);
+	show_result(implode('&', $restype), $tested, $tested_file, $info, $temp_filenames);
 
-	$PHP_FAILED_TESTS[$restype.'ED'][] = array (
+	foreach ($restype as $type) {
+		$PHP_FAILED_TESTS[$type.'ED'][] = array (
 						'name' => $file,
 						'test_name' => (is_array($IN_REDIRECT) ? $IN_REDIRECT['via'] : '') . $tested . " [$tested_file]",
 						'output' => $output_filename,
 						'diff'   => $diff_filename,
 						'info'   => $info,
 						);
+	}
 
 	if (isset($old_php)) {
 		$php = $old_php;
 	}
 
-	return $restype.'ED';
+	return $restype[0].'ED';
 }
 
 function comp_line($l1,$l2,$is_reg)
@@ -1689,7 +1727,14 @@ function settings2array($settings, &$ini_settings)
 			$setting = explode("=", $setting, 2);
 			$name = trim(strtolower($setting[0]));
 			$value = trim($setting[1]);
-			$ini_settings[$name] = $value;
+			if ($name == 'extension') {
+				if (!isset($ini_settings[$name])) {
+					$ini_settings[$name] = array();
+				}
+				$ini_settings[$name][] = $value;
+			} else {
+				$ini_settings[$name] = $value;
+			}
 		}
 	}
 }
@@ -1698,8 +1743,15 @@ function settings2params(&$ini_settings)
 {
 	$settings = '';
 	foreach($ini_settings as $name => $value) {
-		$value = addslashes($value);
-		$settings .= " -d \"$name=$value\"";
+		if (is_array($value)) {
+			foreach($value as $val) {
+				$val = addslashes($val);
+				$settings .= " -d \"$name=$val\"";
+			}
+		} else {
+			$value = addslashes($value);
+			$settings .= " -d \"$name=$value\"";
+		}
 	}
 	$ini_settings = $settings;
 }
